@@ -1,7 +1,10 @@
 #!/usr/bin/qsh
+#   Copyright (C) 2000-2005, International Business Machines
+#   Corporation and others.  All Rights Reserved.
 #
 # Authors:
 # Ami Fixler
+# Barry Novinger
 # Steven R. Loomis <srl@jtcsv.com>
 # George Rhoten
 #
@@ -13,7 +16,8 @@
 
 #binary_suffixes='ico ICO bmp BMP jpg JPG gif GIF brk BRK'
 #ICU specific binary files
-binary_suffixes='brk BRK bin BIN'
+binary_suffixes='brk BRK bin BIN res RES cnv CNV dat DAT icu ICU spp SPP'
+data_files='icu/source/data/brkitr/* icu/source/data/locales/* icu/source/data/coll/* icu/source/data/rbnf/* icu/source/data/mappings/* icu/source/data/misc/* icu/source/data/translit/* icu/source/data/unidata/* icu/source/test/testdata/*'
 
 usage()
 {
@@ -35,16 +39,27 @@ fi
 echo ""
 echo "Extracting from $1 ..."
 echo ""
+
+# determine which directories in the data_files list
+# are included in the provided archive
+for data_dir in $data_files
+do
+    if (pax -f $1 $data_dir >/dev/null 2>&1)
+    then
+        ebcdic_data="$ebcdic_data `echo $data_dir`";
+    fi
+done
+
 # extract everything as iso-8859-1 except these directories
-pax -C 819 -rcvf $1 icu/data/* icu/source/test/testdata/*
+pax -C 819 -rcvf $1 $ebcdic_data
 
 # extract files while converting them to EBCDIC
 echo ""
 echo "Extracting files which must be in ibm-37 ..."
 echo ""
-pax -C 37 -rvf $1 icu/data/* icu/source/test/testdata/*
+pax -C 37 -rvf $1 $ebcdic_data
 
-if [ $# -gt 1 ]; then 
+if [ $# -gt 1 ]; then
   if [ $2 -eq strip ]; then
     echo ""
     echo "Stripping hex 0d characters ..."
@@ -87,15 +102,20 @@ for file in `find ./icu \( -name \*.txt -print \)`; do
           cut -f2-4 -d ' '|\
           tr 'A-Z' 'a-z'`;
 #    echo "bom8 is" $bom8 "for" $file
-#    bom8=`head -c 3 $file|od -t x1|head -n 1|cut -d ' ' -f2-4`;
     #Find a converted UTF-8 BOM
-    if [ "$bom8" = "057 08b 0ab" ]
+    if [ "$bom8" = "057 08b 0ab" -o "$bom8" = "57 8b ab" ]
     then
-        binary_files="$binary_files `echo $file | cut -d / -f2-`";
+        if [ `echo $binary_files1 | wc -w` -lt 250 ]
+        then
+            binary_files1="$binary_files1 `echo $file | cut -d / -f2-`";
+        else
+            binary_files2="$binary_files2 `echo $file | cut -d / -f2-`";
+        fi
     fi
 done
 
-#echo $binary_files
+#echo $binary_files1
+#echo $binary_files2
 
 for i in $(pax -f $1 2>/dev/null)
 do
@@ -110,7 +130,7 @@ do
        suf=${i#*.*}
        if [ "$suf" = "$j" ]
        then
-         binary_files="$binary_files $i"
+         binary_files2="$binary_files2 $i"
          break
        fi
      done
@@ -122,14 +142,16 @@ do
 done
 
 # now see if a re-extract of binary files is necessary
-if [ ${#binary_files} -eq 0 ]; then
+if [ ${#binary_files1} -eq 0 -a ${#binary_files2} -eq 0 ]; then
   echo ""
   echo "There are no binary files to restore."
 else
   echo "Restoring binary files ..."
   echo ""
-  rm $binary_files
-  pax -C 819 -rvf $1 $binary_files
+  rm $binary_files1
+  rm $binary_files2
+  pax -C 819 -rvf $1 $binary_files1
+  pax -C 819 -rvf $1 $binary_files2
 fi
 echo ""
 echo "$0 has completed extracting ICU from $1."
