@@ -1,43 +1,46 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1999-2001, International Business Machines
+*   Copyright (C) 1999-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************/
 
 
-/*----------------------------------------------------------------------------------
+/*----------------------------------------------------------------------------
  *                           
  *       Memory mapped file wrappers for use by the ICU Data Implementation
  *       All of the platform-specific implementation for mapping data files
  *         is here.  The rest of the ICU Data implementation uses only the
  *         wrapper functions.
  *
- *----------------------------------------------------------------------------------*/
-#include "unicode/utypes.h"
+ *----------------------------------------------------------------------------*/
+#include "unicode/putil.h"
 
 
 #include "udatamem.h"
 #include "umapfile.h"
 
-
 /* memory-mapping base definitions ------------------------------------------ */
 
-
+/* MAP_NONE: no memory mapping, no file access at all */
+#define MAP_NONE        0
 #define MAP_WIN32       1
 #define MAP_POSIX       2
-#define MAP_FILE_STREAM 3
+#define MAP_STDIO       3
 #define MAP_390DLL      4
 
-#ifdef WIN32
+#if UCONFIG_NO_FILE_IO
+#   define MAP_IMPLEMENTATION MAP_NONE
+#elif defined(U_WINDOWS)
 #   define WIN32_LEAN_AND_MEAN
-#   define NOGDI
+#   define VC_EXTRALEAN
 #   define NOUSER
 #   define NOSERVICE
 #   define NOIME
 #   define NOMCX
 #   include <windows.h>
+#   include "cmemory.h"
 
     typedef HANDLE MemoryMap;
 
@@ -46,11 +49,16 @@
 #   define MAP_IMPLEMENTATION MAP_WIN32
 
 /* ### Todo: properly auto detect mmap(). Until then, just add your platform here. */
-#elif U_HAVE_MMAP || defined(AIX) || defined(HPUX) || defined(OS390) || defined(PTX)
+#elif U_HAVE_MMAP || defined(U_AIX) || defined(U_HPUX) || defined(OS390) || defined(PTX)
     typedef size_t MemoryMap;
 
 #   define IS_MAP(map) ((map)!=0)
 
+    /* Needed by OSF to get the correct mmap version */
+#   ifndef _XOPEN_SOURCE_EXTENDED
+#       define _XOPEN_SOURCE_EXTENDED
+#   endif
+
 #   include <unistd.h>
 #   include <sys/mman.h>
 #   include <sys/stat.h>
@@ -60,45 +68,55 @@
 #       define MAP_FAILED ((void*)-1)
 #   endif
 
-#   ifdef OS390
+#   if defined(OS390) && defined (OS390_STUBDATA)
         /*   No memory mapping for 390 batch mode.  Fake it using dll loading.  */
 #       include <dll.h>
 #       include "cstring.h"
 #       include "cmemory.h"
 #       include "unicode/udata.h"
 #       define LIB_PREFIX "lib"
-#       define LIB_PREFIX_LENGTH 3
 #       define LIB_SUFFIX ".dll"
 #       define MAP_IMPLEMENTATION MAP_390DLL
 
 /* This is inconvienient until we figure out what to do with U_ICUDATA_NAME in utypes.h */
-#       define U_ICUDATA_ENTRY_NAME "icudt" U_ICU_VERSION_SHORT U_ICU_LIB_SUFFIX_C_NAME_STRING "_dat"
+#       define U_ICUDATA_ENTRY_NAME "icudt" U_ICU_VERSION_SHORT U_LIB_SUFFIX_C_NAME_STRING "_dat"
 #   else
 #       define MAP_IMPLEMENTATION MAP_POSIX
 #   endif
 
-#else /* unknown platform, no memory map implementation: use FileStream/uprv_malloc() instead */
+#else /* unknown platform, no memory map implementation: use stdio.h and uprv_malloc() instead */
 
-#   include "filestrm.h"
+#   include <stdio.h>
+#   include "cmemory.h"
 
     typedef void *MemoryMap;
 
 #   define IS_MAP(map) ((map)!=NULL)
 
-#   define MAP_IMPLEMENTATION MAP_FILE_STREAM
+#   define MAP_IMPLEMENTATION MAP_STDIO
 
 #endif
 
 
 
 
-/*----------------------------------------------------------------------------------*
- *                                                                                  *
- *   Memory Mapped File support.  Platform dependent implementation of functions    *
- *                                used by the rest of the implementation.           *
- *                                                                                  *
- *----------------------------------------------------------------------------------*/
-#if MAP_IMPLEMENTATION==MAP_WIN32
+/*----------------------------------------------------------------------------*
+ *                                                                            *
+ *   Memory Mapped File support.  Platform dependent implementation of        *
+ *                           functions used by the rest of the implementation.*
+ *                                                                            *
+ *----------------------------------------------------------------------------*/
+#if MAP_IMPLEMENTATION==MAP_NONE
+    UBool
+    uprv_mapFile(UDataMemory *pData, const char *path) {
+        UDataMemory_init(pData); /* Clear the output struct. */
+        return FALSE;            /* no file access */
+    }
+
+    void uprv_unmapFile(UDataMemory *pData) {
+        /* nothing to do */
+    }
+#elif MAP_IMPLEMENTATION==MAP_WIN32
     UBool
     uprv_mapFile(
          UDataMemory *pData,    /* Fill in with info on the result doing the mapping. */
@@ -108,6 +126,9 @@
     {
         HANDLE map;
         HANDLE file;
+        SECURITY_ATTRIBUTES mappingAttributes;
+        SECURITY_ATTRIBUTES *mappingAttributesPtr = NULL;
+        SECURITY_DESCRIPTOR securityDesc;
 
         UDataMemory_init(pData); /* Clear the output struct.        */
 
@@ -119,8 +140,24 @@
             return FALSE;
         }
 
+        /* Declare and initialize a security descriptor.
+           This is required for multiuser systems on Windows 2000 SP4 and beyond */
+        if (InitializeSecurityDescriptor(&securityDesc, SECURITY_DESCRIPTOR_REVISION)) {
+            /* give the security descriptor a Null Dacl done using the  "TRUE, (PACL)NULL" here	*/
+            if (SetSecurityDescriptorDacl(&securityDesc, TRUE, (PACL)NULL, FALSE)) {
+                /* Make the security attributes point to the security descriptor */
+                uprv_memset(&mappingAttributes, 0, sizeof(mappingAttributes));
+                mappingAttributes.nLength = sizeof(mappingAttributes);
+                mappingAttributes.lpSecurityDescriptor = &securityDesc;
+                mappingAttributes.bInheritHandle = FALSE; /* object uninheritable */
+                mappingAttributesPtr = &mappingAttributes;
+            }
+        }
+        /* else creating security descriptors can fail when we are on Windows 98,
+           and mappingAttributesPtr == NULL for that case. */
+
         /* create an unnamed Windows file-mapping object for the specified file */
-        map=CreateFileMapping(file, NULL, PAGE_READONLY, 0, 0, NULL);
+        map=CreateFileMapping(file, mappingAttributesPtr, PAGE_READONLY, 0, 0, NULL);
         CloseHandle(file);
         if(map==NULL) {
             return FALSE;
@@ -172,7 +209,7 @@
         }
 
         /* get a view of the mapping */
-#ifndef HPUX
+#ifndef U_HPUX
         data=mmap(0, length, PROT_READ, MAP_SHARED,  fd, 0);
 #else
         data=mmap(0, length, PROT_READ, MAP_PRIVATE, fd, 0);
@@ -204,49 +241,63 @@
 
 
 
-#elif MAP_IMPLEMENTATION==MAP_FILE_STREAM
-    static UBool
+#elif MAP_IMPLEMENTATION==MAP_STDIO
+    /* copy of the filestrm.c/T_FileStream_size() implementation */
+    static int32_t
+    umap_fsize(FILE *f) {
+        int32_t savedPos = ftell(f);
+        int32_t size = 0;
+
+        /*Changes by Bertrand A. D. doesn't affect the current position
+        goes to the end of the file before ftell*/
+        fseek(f, 0, SEEK_END);
+        size = (int32_t)ftell(f);
+        fseek(f, savedPos, SEEK_SET);
+        return size;
+    }
+
+    UBool
     uprv_mapFile(UDataMemory *pData, const char *path) {
-        FileStream *file;
+        FILE *file;
         int32_t fileLength;
         void *p;
 
         UDataMemory_init(pData); /* Clear the output struct.        */
         /* open the input file */
-        file=T_FileStream_open(path, "rb");
+        file=fopen(path, "rb");
         if(file==NULL) {
             return FALSE;
         }
 
         /* get the file length */
-        fileLength=T_FileStream_size(file);
-        if(T_FileStream_error(file) || fileLength<=20) {
-            T_FileStream_close(file);
+        fileLength=umap_fsize(file);
+        if(ferror(file) || fileLength<=20) {
+            fclose(file);
             return FALSE;
         }
 
         /* allocate the memory to hold the file data */
         p=uprv_malloc(fileLength);
         if(p==NULL) {
-            T_FileStream_close(file);
+            fclose(file);
             return FALSE;
         }
 
         /* read the file */
-        if(fileLength!=T_FileStream_read(file, p, fileLength)) {
+        if(fileLength!=fread(p, 1, fileLength, file)) {
             uprv_free(p);
-            T_FileStream_close(file);
+            fclose(file);
             return FALSE;
         }
 
-        T_FileStream_close(file);
+        fclose(file);
         pData->map=p;
         pData->pHeader=(const DataHeader *)p;
         pData->mapAddr=p;
         return TRUE;
     }
 
-    static void
+    void
     uprv_unmapFile(UDataMemory *pData) {
         if(pData!=NULL && pData->map!=NULL) {
             uprv_free(pData->map);
@@ -268,16 +319,79 @@
      *
      */
 
+    static char *strcpy_returnEnd(char *dest, const char *src)
+    {
+        while((*dest=*src)!=0) {
+            ++dest;
+            ++src;
+        }
+        return dest;
+    }
+    
+    /*------------------------------------------------------------------------------
+     *                                                                              
+     *  computeDirPath   given a user-supplied path of an item to be opened,             
+     *                         compute and return 
+     *                            - the full directory path to be used 
+     *                              when opening the file.
+     *                            - Pointer to null at end of above returned path    
+     *
+     *                       Parameters:
+     *                          path:        input path.  Buffer is not altered.
+     *                          pathBuffer:  Output buffer.  Any contents are overwritten.
+     *
+     *                       Returns:
+     *                          Pointer to null termination in returned pathBuffer.
+     *
+     *                    TODO:  This works the way ICU historically has, but the
+     *                           whole data fallback search path is so complicated that
+     *                           proabably almost no one will ever really understand it,
+     *                           the potential for confusion is large.  (It's not just 
+     *                           this one function, but the whole scheme.)
+     *                            
+     *------------------------------------------------------------------------------*/
+    static char *uprv_computeDirPath(const char *path, char *pathBuffer)
+    {
+        char   *finalSlash;       /* Ptr to last dir separator in input path, or null if none. */
+        int32_t pathLen;          /* Length of the returned directory path                     */
+        
+        finalSlash = 0;
+        if (path != 0) {
+            finalSlash = uprv_strrchr(path, U_FILE_SEP_CHAR);
+        }
+        
+        *pathBuffer = 0;
+        if (finalSlash == 0) {
+        /* No user-supplied path.  
+            * Copy the ICU_DATA path to the path buffer and return that*/
+            const char *icuDataDir;
+            icuDataDir=u_getDataDirectory();
+            if(icuDataDir!=NULL && *icuDataDir!=0) {
+                return strcpy_returnEnd(pathBuffer, icuDataDir);
+            } else {
+                /* there is no icuDataDir either.  Just return the empty pathBuffer. */
+                return pathBuffer;
+            }
+        } 
+        
+        /* User supplied path did contain a directory portion.
+        * Copy it to the output path buffer */
+        pathLen = (int32_t)(finalSlash - path + 1);
+        uprv_memcpy(pathBuffer, path, pathLen);
+        *(pathBuffer+pathLen) = 0;
+        return pathBuffer+pathLen;
+    }
+    
+
 #   define DATA_TYPE "dat"
 
     UBool uprv_mapFile(UDataMemory *pData, const char *path) {
         const char *inBasename;
-        char *basename, *suffix, *tempbasename;
+        char *basename;
         char pathBuffer[1024];
         const DataHeader *pHeader;
         dllhandle *handle;
-        char filename[1024];
-        void *val=0; 
+        void *val=0;
 
         inBasename=uprv_strrchr(path, U_FILE_SEP_CHAR);
         if(inBasename==NULL) {
@@ -319,26 +433,19 @@
         }
 
 #       ifdef OS390BATCH
-        /* ### hack: we still need to get u_getDataDirectory() fixed
-        for OS/390 (batch mode - always return "//"? )
-        and this here straightened out with LIB_PREFIX and LIB_SUFFIX (both empty?!)
-        This is probably due to the strange file system on OS/390.  It's more like
-        a database with short entry names than a typical file system. */
+            /* ### hack: we still need to get u_getDataDirectory() fixed
+            for OS/390 (batch mode - always return "//"? )
+            and this here straightened out with LIB_PREFIX and LIB_SUFFIX (both empty?!)
+            This is probably due to the strange file system on OS/390.  It's more like
+            a database with short entry names than a typical file system. */
             /* U_ICUDATA_NAME should always have the correct name */
-            /* 390port: BUT FOR BATCH MODE IT IS AN EXCEPTION ... */
-            /* 390port: THE NEXT LINE OF CODE WILL NOT WORK !!!!! */
-            /*lib=LOAD_LIBRARY("//" U_ICUDATA_NAME, "//" U_ICUDATA_NAME);*/
-            uprv_strcpy(pathBuffer, "//IXMI" U_ICU_VERSION_SHORT "DA");
+            /* BUT FOR BATCH MODE IT IS AN EXCEPTION BECAUSE */
+            /* THE FIRST THREE LETTERS ARE PREASSIGNED TO THE */
+            /* PROJECT!!!!! */
+            uprv_strcpy(pathBuffer, "IXMI" U_ICU_VERSION_SHORT "DA");
 #       else
             /* set up the library name */
-            uprv_memcpy(basename, LIB_PREFIX, LIB_PREFIX_LENGTH);
-            inBasename = U_ICUDATA_NAME;
-            suffix = basename+LIB_PREFIX_LENGTH;
-            while((*suffix=*inBasename)!=0) {
-                ++suffix;
-                ++inBasename;
-            }
-            uprv_strcpy(suffix, LIB_SUFFIX);
+            uprv_strcpy(basename, LIB_PREFIX U_LIBICUDATA_NAME U_ICU_VERSION_SHORT LIB_SUFFIX);
 #       endif
 
 #       ifdef UDATA_DEBUG
@@ -373,7 +480,7 @@
 
 
 
-    void    uprv_unmapFile(UDataMemory *pData) {
+    void uprv_unmapFile(UDataMemory *pData) {
         if(pData!=NULL && pData->map!=NULL) {
             uprv_free(pData->map);
             pData->map     = NULL;
@@ -385,5 +492,3 @@
 #else
 #   error MAP_IMPLEMENTATION is set incorrectly
 #endif
-
-
