@@ -1,6 +1,6 @@
 /*
 *******************************************************************************
-* Copyright (C) 1996-2001, International Business Machines Corporation and    *
+* Copyright (C) 1996-2003, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 */
@@ -27,6 +27,10 @@
 //                             than the other
 //===============================================================================
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/sortkey.h"
 #include "cmemory.h"
 #include "uhash.h"
@@ -39,15 +43,17 @@ U_NAMESPACE_BEGIN
 #define kInvalidHashCode ((int32_t)0)
 #define kEmptyHashCode ((int32_t)1)
 
+UOBJECT_DEFINE_RTTI_IMPLEMENTATION(CollationKey)
+
 CollationKey::CollationKey()
-    : fBogus(FALSE), fCount(0), fCapacity(0),
+    : UObject(), fBogus(FALSE), fCount(0), fCapacity(0),
       fHashCode(kEmptyHashCode), fBytes(NULL)
 {
 }
 
 // Create a collation key from a bit array.
 CollationKey::CollationKey(const uint8_t* newValues, int32_t count)
-    : fBogus(FALSE), fCount(count), fCapacity(count),
+    : UObject(), fBogus(FALSE), fCount(count), fCapacity(count),
       fHashCode(kInvalidHashCode)
 {
     fBytes = (uint8_t *)uprv_malloc(count);
@@ -62,7 +68,7 @@ CollationKey::CollationKey(const uint8_t* newValues, int32_t count)
 }
 
 CollationKey::CollationKey(const CollationKey& other)
-: fBogus(FALSE), fCount(other.fCount), fCapacity(other.fCapacity),
+: UObject(other), fBogus(FALSE), fCount(other.fCount), fCapacity(other.fCapacity),
   fHashCode(other.fHashCode), fBytes(NULL)
 {
     if (other.fBogus)
@@ -179,7 +185,7 @@ CollationKey::compareTo(const CollationKey& target) const
 
     // are we comparing the same string
     if (src == tgt)
-        return Collator::EQUAL;
+        return  Collator::EQUAL;
 
         /*
         int count = (this->fCount < target.fCount) ? this->fCount : target.fCount;
@@ -208,16 +214,16 @@ CollationKey::compareTo(const CollationKey& target) const
     if (this->fCount != target.fCount) {
         if (this->fCount < target.fCount) {
             minLength = this->fCount;
-            result    = Collator::LESS;
+            result    =  Collator::LESS;
         }
         else {
             minLength = target.fCount;
-            result    = Collator::GREATER;
+            result    =  Collator::GREATER;
         }
     }
     else {
         minLength = target.fCount;
-        result    = Collator::EQUAL;
+        result    =  Collator::EQUAL;
     }
 
     if (minLength > 0) {
@@ -246,6 +252,54 @@ CollationKey::compareTo(const CollationKey& target) const
     */
 }
 
+// Bitwise comparison for the collation keys.
+UCollationResult
+CollationKey::compareTo(const CollationKey& target, UErrorCode &status) const
+{
+  if(U_SUCCESS(status)) {
+    uint8_t *src = this->fBytes;
+    uint8_t *tgt = target.fBytes;
+
+    // are we comparing the same string
+    if (src == tgt)
+        return  UCOL_EQUAL;
+
+    int                         minLength;
+    UCollationResult result;
+
+    // are we comparing different lengths?
+    if (this->fCount != target.fCount) {
+        if (this->fCount < target.fCount) {
+            minLength = this->fCount;
+            result    =  UCOL_LESS;
+        }
+        else {
+            minLength = target.fCount;
+            result    =  UCOL_GREATER;
+        }
+    }
+    else {
+        minLength = target.fCount;
+        result    =  UCOL_EQUAL;
+    }
+
+    if (minLength > 0) {
+        int diff = uprv_memcmp(src, tgt, minLength);
+        if (diff > 0) {
+            return UCOL_GREATER;
+        }
+        else
+            if (diff < 0) {
+                return UCOL_LESS;
+            }
+    }
+
+    return result;
+  } else {
+    return UCOL_EQUAL;
+  }
+}
+
 CollationKey&
 CollationKey::ensureCapacity(int32_t newSize)
 {
@@ -271,6 +325,7 @@ CollationKey::ensureCapacity(int32_t newSize)
     return *this;
 }
 
+#ifdef U_USE_COLLATION_KEY_DEPRECATES
 // Create a copy of the byte array.
 uint8_t*
 CollationKey::toByteArray(int32_t& count) const
@@ -289,6 +344,7 @@ CollationKey::toByteArray(int32_t& count) const
 
     return result;
 }
+#endif
 
 int32_t
 CollationKey::hashCode() const
@@ -335,3 +391,5 @@ CollationKey::hashCode() const
 }
 
 U_NAMESPACE_END
+
+#endif /* #if !UCONFIG_NO_COLLATION */
