@@ -1,7 +1,7 @@
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2001, International Business Machines
+*   Copyright (C) 2001-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
@@ -23,19 +23,23 @@
 *   03/07/2001  synwee     merged UCA's maxexpansion and tailoring's
 */
 
-#include "ucol_elm.h"
-#include "unicode/uchar.h"
-#include "unormimp.h"
-#include "cmemory.h"
-#include "unicode/unistr.h"
-#include "caniter.h"
+#include "unicode/utypes.h"
 
-U_NAMESPACE_BEGIN
+#if !UCONFIG_NO_COLLATION
+
+#include "unicode/uchar.h"
+#include "unicode/unistr.h"
+#include "unicode/ucoleitr.h"
+#include "unicode/normlzr.h"
+#include "ucol_elm.h"
+#include "unormimp.h"
+#include "unicode/caniter.h"
+#include "cmemory.h"
 
 static uint32_t uprv_uca_processContraction(CntTable *contractions, UCAElements *element, uint32_t existingCE, UErrorCode *status);
 
 U_CDECL_BEGIN
-static int32_t U_EXPORT2 U_CALLCONV
+static int32_t U_CALLCONV
 prefixLookupHash(const UHashTok e) {
   UCAElements *element = (UCAElements *)e.pointer;
   UChar buf[256];
@@ -48,7 +52,7 @@ prefixLookupHash(const UHashTok e) {
   return uhash_hashUChars(key);
 }
 
-static int8_t U_EXPORT2 U_CALLCONV
+static int8_t U_CALLCONV
 prefixLookupComp(const UHashTok e1, const UHashTok e2) {
   UCAElements *element1 = (UCAElements *)e1.pointer;
   UCAElements *element2 = (UCAElements *)e2.pointer;
@@ -65,34 +69,21 @@ prefixLookupComp(const UHashTok e1, const UHashTok e2) {
   uprv_memcpy(buf2, element2->cPoints, element2->cSize*sizeof(UChar));
   buf2[element2->cSize] = 0;
 
-  //key1.pointer = element1->cPoints;
-  //key2.pointer = element2->cPoints;
-  //element1->cPoints[element1->cSize] = 0;
-  //element2->cPoints[element2->cSize] = 0;
-
   return uhash_compareUChars(key1, key2);
 }
 U_CDECL_END
 
-/*
-static void uprv_uca_reverseElement(UCAElements *el) {
-    uint32_t i = 0;
-    UChar temp;
-
-    for(i = 0; i<el->cSize/2; i++) {
-        temp = el->cPoints[i];
-        el->cPoints[i] = el->cPoints[el->cSize-i-1];
-        el->cPoints[el->cSize-i-1] = temp;
-    }
-}
-*/
-
 static int32_t uprv_uca_addExpansion(ExpansionTable *expansions, uint32_t value, UErrorCode *status) {
     if(U_FAILURE(*status)) {
         return 0;
     }
     if(expansions->CEs == NULL) {
         expansions->CEs = (uint32_t *)uprv_malloc(INIT_EXP_TABLE_SIZE*sizeof(uint32_t));
+        /* test for NULL */
+        if (expansions->CEs == NULL) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
+            return 0;
+        }
         expansions->size = INIT_EXP_TABLE_SIZE;
         expansions->position = 0;
     }
@@ -114,26 +105,52 @@ static int32_t uprv_uca_addExpansion(ExpansionTable *expansions, uint32_t value,
     return(expansions->position++);
 }
 
-static inline void U_CALLCONV
-uhash_freeBlockWrapper(void *obj) {
-  uhash_freeBlock(obj);
-}
-
 U_CAPI tempUCATable*  U_EXPORT2
-uprv_uca_initTempTable(UCATableHeader *image, UColOptionSet *opts, const UCollator *UCA, UColCETags initTag, UErrorCode *status) {
+uprv_uca_initTempTable(UCATableHeader *image, UColOptionSet *opts, const UCollator *UCA, UColCETags initTag, UColCETags supplementaryInitTag, UErrorCode *status) {
   tempUCATable *t = (tempUCATable *)uprv_malloc(sizeof(tempUCATable));
+  /* test for NULL */
+  if (t == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return NULL;
+  }
   MaxExpansionTable *maxet  = (MaxExpansionTable *)uprv_malloc(
                                                    sizeof(MaxExpansionTable));
+  /* test for NULL */
+  if (maxet == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(t);
+      return NULL;
+  }
   MaxJamoExpansionTable *maxjet = (MaxJamoExpansionTable *)uprv_malloc(
                                                sizeof(MaxJamoExpansionTable));
+  /* test for NULL */
+  if (maxjet == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(t);
+      uprv_free(maxet);
+      return NULL;
+  }
   t->image = image;
   t->options = opts;
 
   t->UCA = UCA;
   t->expansions = (ExpansionTable *)uprv_malloc(sizeof(ExpansionTable));
+  /* test for NULL */
+  if (t->expansions == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(t);
+      uprv_free(maxet);
+      uprv_free(maxjet);
+      return NULL;
+  }
   uprv_memset(t->expansions, 0, sizeof(ExpansionTable));
   /*t->mapping = ucmpe32_open(UCOL_SPECIAL_FLAG | (initTag<<24), UCOL_SPECIAL_FLAG | (SURROGATE_TAG<<24), UCOL_SPECIAL_FLAG | (LEAD_SURROGATE_TAG<<24), status);*/
-  t->mapping = utrie_open(NULL, NULL, 0x100000, UCOL_SPECIAL_FLAG | (initTag<<24), TRUE); // Do your own mallocs for the structure, array and have linear Latin 1
+  /*t->mapping = utrie_open(NULL, NULL, 0x100000, UCOL_SPECIAL_FLAG | (initTag<<24), TRUE); // Do your own mallocs for the structure, array and have linear Latin 1*/
+
+  t->mapping = utrie_open(NULL, NULL, 0x100000,
+                          UCOL_SPECIAL_FLAG | (initTag<<24),
+                          UCOL_SPECIAL_FLAG | (supplementaryInitTag << 24),
+                          TRUE); // Do your own mallocs for the structure, array and have linear Latin 1
   t->prefixLookup = uhash_open(prefixLookupHash, prefixLookupComp, status);
   uhash_setValueDeleter(t->prefixLookup, uhash_freeBlock);
 
@@ -148,8 +165,19 @@ uprv_uca_initTempTable(UCATableHeader *image, UColOptionSet *opts, const UCollat
     maxet->position        = maxet->size - 1;
     maxet->endExpansionCE  = 
                       (uint32_t *)uprv_malloc(sizeof(uint32_t) * maxet->size);
+    /* test for NULL */
+    if (maxet->endExpansionCE == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     maxet->expansionCESize =
                         (uint8_t *)uprv_malloc(sizeof(uint8_t) * maxet->size);
+    /* test for NULL */
+    if (maxet->expansionCESize == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        uprv_free(maxet->endExpansionCE);
+        return NULL;
+    }
     /* initialized value */
     *(maxet->endExpansionCE)  = 0;
     *(maxet->expansionCESize) = 0;
@@ -171,7 +199,18 @@ uprv_uca_initTempTable(UCATableHeader *image, UColOptionSet *opts, const UCollat
   maxjet->maxTSize = 1;
 
   t->unsafeCP = (uint8_t *)uprv_malloc(UCOL_UNSAFECP_TABLE_SIZE);
+  /* test for NULL */
+  if (t->unsafeCP == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      return NULL;
+  }
   t->contrEndCP = (uint8_t *)uprv_malloc(UCOL_UNSAFECP_TABLE_SIZE);
+  /* test for NULL */
+  if (t->contrEndCP == NULL) {
+      *status = U_MEMORY_ALLOCATION_ERROR;
+      uprv_free(t->unsafeCP);
+      return NULL;
+  }
   uprv_memset(t->unsafeCP, 0, UCOL_UNSAFECP_TABLE_SIZE);
   uprv_memset(t->contrEndCP, 0, UCOL_UNSAFECP_TABLE_SIZE);
 return t;
@@ -184,6 +223,11 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
   }
 
   tempUCATable *r = (tempUCATable *)uprv_malloc(sizeof(tempUCATable));
+  /* test for NULL */
+  if (r == NULL) {
+    *status = U_MEMORY_ALLOCATION_ERROR;
+    return NULL;
+  }
   uprv_memset(r, 0, sizeof(tempUCATable));
 
   /* mapping */
@@ -199,11 +243,21 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
   /* expansions */
   if(t->expansions != NULL) {
     r->expansions = (ExpansionTable *)uprv_malloc(sizeof(ExpansionTable));
+    /* test for NULL */
+    if (r->expansions == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     r->expansions->position = t->expansions->position;
     r->expansions->size = t->expansions->size;
     if(t->expansions->CEs != NULL) {
       r->expansions->CEs = (uint32_t *)uprv_malloc(sizeof(uint32_t)*t->expansions->size);
-      uprv_memcpy(r->expansions->CEs, t->expansions->CEs, sizeof(uint32_t)*t->expansions->size);
+      /* test for NULL */
+      if (r->expansions->CEs == NULL) {
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          return NULL;
+      }
+      uprv_memcpy(r->expansions->CEs, t->expansions->CEs, sizeof(uint32_t)*t->expansions->position);
     } else {
       r->expansions->CEs = NULL;
     }
@@ -216,17 +270,34 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
 
   if(t->maxExpansions != NULL) {
     r->maxExpansions = (MaxExpansionTable *)uprv_malloc(sizeof(MaxExpansionTable));
+    /* test for NULL */
+    if (r->maxExpansions == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     r->maxExpansions->size = t->maxExpansions->size;
     r->maxExpansions->position = t->maxExpansions->position;
     if(t->maxExpansions->endExpansionCE != NULL) {
       r->maxExpansions->endExpansionCE = (uint32_t *)uprv_malloc(sizeof(uint32_t)*t->maxExpansions->size);
-      uprv_memcpy(r->maxExpansions->endExpansionCE, t->maxExpansions->endExpansionCE, t->maxExpansions->size*sizeof(uint32_t));
+      uprv_memset(r->maxExpansions->endExpansionCE, 0xDB, sizeof(uint32_t)*t->maxExpansions->size);
+      /* test for NULL */
+      if (r->maxExpansions->endExpansionCE == NULL) {
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          return NULL;
+      }
+      uprv_memcpy(r->maxExpansions->endExpansionCE, t->maxExpansions->endExpansionCE, t->maxExpansions->position*sizeof(uint32_t));
     } else {
       r->maxExpansions->endExpansionCE = NULL;
     }
     if(t->maxExpansions->expansionCESize != NULL) {
       r->maxExpansions->expansionCESize = (uint8_t *)uprv_malloc(sizeof(uint8_t)*t->maxExpansions->size);
-      uprv_memcpy(r->maxExpansions->expansionCESize, t->maxExpansions->expansionCESize, t->maxExpansions->size*sizeof(uint8_t));
+      uprv_memset(r->maxExpansions->expansionCESize, 0xDB, sizeof(uint8_t)*t->maxExpansions->size);
+      /* test for NULL */
+      if (r->maxExpansions->expansionCESize == NULL) {
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          return NULL;
+      }
+      uprv_memcpy(r->maxExpansions->expansionCESize, t->maxExpansions->expansionCESize, t->maxExpansions->position*sizeof(uint8_t));
     } else {
       r->maxExpansions->expansionCESize = NULL;
     }
@@ -234,6 +305,11 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
 
   if(t->maxJamoExpansions != NULL) {
     r->maxJamoExpansions = (MaxJamoExpansionTable *)uprv_malloc(sizeof(MaxJamoExpansionTable));
+    /* test for NULL */
+    if (r->maxJamoExpansions == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     r->maxJamoExpansions->size = t->maxJamoExpansions->size;
     r->maxJamoExpansions->position = t->maxJamoExpansions->position;
     r->maxJamoExpansions->maxLSize = t->maxJamoExpansions->maxLSize;
@@ -241,9 +317,19 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
     r->maxJamoExpansions->maxTSize = t->maxJamoExpansions->maxTSize;
     if(t->maxJamoExpansions->size != 0) {
       r->maxJamoExpansions->endExpansionCE = (uint32_t *)uprv_malloc(sizeof(uint32_t)*t->maxJamoExpansions->size);
-      uprv_memcpy(r->maxJamoExpansions->endExpansionCE, t->maxJamoExpansions->endExpansionCE, t->maxJamoExpansions->size*sizeof(uint32_t));
+      /* test for NULL */
+      if (r->maxJamoExpansions->endExpansionCE == NULL) {
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          return NULL;
+      }
+      uprv_memcpy(r->maxJamoExpansions->endExpansionCE, t->maxJamoExpansions->endExpansionCE, t->maxJamoExpansions->position*sizeof(uint32_t));
       r->maxJamoExpansions->isV = (UBool *)uprv_malloc(sizeof(UBool)*t->maxJamoExpansions->size);
-      uprv_memcpy(r->maxJamoExpansions->isV, t->maxJamoExpansions->isV, t->maxJamoExpansions->size*sizeof(UBool));
+      /* test for NULL */
+      if (r->maxJamoExpansions->isV == NULL) {
+          *status = U_MEMORY_ALLOCATION_ERROR;
+          return NULL;
+      }
+      uprv_memcpy(r->maxJamoExpansions->isV, t->maxJamoExpansions->isV, t->maxJamoExpansions->position*sizeof(UBool));
     } else {
       r->maxJamoExpansions->endExpansionCE = NULL;
       r->maxJamoExpansions->isV = NULL;
@@ -252,11 +338,21 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
 
   if(t->unsafeCP != NULL) {
     r->unsafeCP = (uint8_t *)uprv_malloc(UCOL_UNSAFECP_TABLE_SIZE);
+    /* test for NULL */
+    if (r->unsafeCP == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     uprv_memcpy(r->unsafeCP, t->unsafeCP, UCOL_UNSAFECP_TABLE_SIZE);
   }
 
   if(t->contrEndCP != NULL) {
     r->contrEndCP = (uint8_t *)uprv_malloc(UCOL_UNSAFECP_TABLE_SIZE);
+    /* test for NULL */
+    if (r->contrEndCP == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
     uprv_memcpy(r->contrEndCP, t->contrEndCP, UCOL_UNSAFECP_TABLE_SIZE);
   }
 
@@ -270,32 +366,34 @@ uprv_uca_cloneTempTable(tempUCATable *t, UErrorCode *status) {
 
 U_CAPI void  U_EXPORT2
 uprv_uca_closeTempTable(tempUCATable *t) {
-  uprv_free(t->expansions->CEs);
-  uprv_free(t->expansions);
-  if(t->contractions != NULL) {
-    uprv_cnttab_close(t->contractions);
+  if(t != NULL) {
+    uprv_free(t->expansions->CEs);
+    uprv_free(t->expansions);
+    if(t->contractions != NULL) {
+      uprv_cnttab_close(t->contractions);
+    }
+    /*ucmpe32_close(t->mapping);*/
+    utrie_close(t->mapping);
+
+    if(t->prefixLookup != NULL) {
+      uhash_close(t->prefixLookup);
+    }
+
+    uprv_free(t->maxExpansions->endExpansionCE);
+    uprv_free(t->maxExpansions->expansionCESize);
+    uprv_free(t->maxExpansions);
+
+    if (t->maxJamoExpansions->size > 0) {
+      uprv_free(t->maxJamoExpansions->endExpansionCE);
+      uprv_free(t->maxJamoExpansions->isV);
+    }
+    uprv_free(t->maxJamoExpansions);
+
+    uprv_free(t->unsafeCP);
+    uprv_free(t->contrEndCP);
+
+    uprv_free(t);
   }
-  /*ucmpe32_close(t->mapping);*/
-  utrie_close(t->mapping);
-
-  if(t->prefixLookup != NULL) {
-    uhash_close(t->prefixLookup);
-  }
-
-  uprv_free(t->maxExpansions->endExpansionCE);
-  uprv_free(t->maxExpansions->expansionCESize);
-  uprv_free(t->maxExpansions);
-
-  if (t->maxJamoExpansions->size > 0) {
-    uprv_free(t->maxJamoExpansions->endExpansionCE);
-    uprv_free(t->maxJamoExpansions->isV);
-  }
-  uprv_free(t->maxJamoExpansions);
-
-  uprv_free(t->unsafeCP);
-  uprv_free(t->contrEndCP);
-
-  uprv_free(t);
 }
 
 /**
@@ -308,7 +406,7 @@ uprv_uca_closeTempTable(tempUCATable *t) {
 * @param status error status
 * @returns size of the maxexpansion and maxsize used.
 */
-int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
+static int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
                              uint8_t            expansionsize,
                              MaxExpansionTable *maxexpansion,
                              UErrorCode        *status)
@@ -317,9 +415,19 @@ int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
     /* we'll always make the first element 0, for easier manipulation */
     maxexpansion->endExpansionCE = 
                (uint32_t *)uprv_malloc(INIT_EXP_TABLE_SIZE * sizeof(int32_t));
+    /* test for NULL */
+    if (maxexpansion->endExpansionCE == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
     *(maxexpansion->endExpansionCE) = 0;
     maxexpansion->expansionCESize =
                (uint8_t *)uprv_malloc(INIT_EXP_TABLE_SIZE * sizeof(uint8_t));
+    /* test for NULL */;
+    if (maxexpansion->expansionCESize == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
     *(maxexpansion->expansionCESize) = 0;
     maxexpansion->size     = INIT_EXP_TABLE_SIZE;
     maxexpansion->position = 0;
@@ -362,7 +470,7 @@ int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
       start = mid;                                                           
     }                                                                        
   } 
-      
+
   if (*start == endexpansion) {                                                     
     result = start - pendexpansionce;  
   }                                                                          
@@ -370,7 +478,7 @@ int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
     if (*limit == endexpansion) {                                                     
       result = limit - pendexpansionce;      
     }                                            
-      
+
   if (result > -1) {
     /* found the ce in expansion, we'll just modify the size if it is 
        smaller */
@@ -386,9 +494,9 @@ int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
     int      shiftsize     = (pendexpansionce + pos) - start;
     uint32_t *shiftpos     = start + 1;
     uint8_t  *sizeshiftpos = pexpansionsize + (shiftpos - pendexpansionce);
-    
+
     /* okay need to rearrange the array into sorted order */
-    if (shiftsize == 0 || *(pendexpansionce + pos) < endexpansion) {
+    if (shiftsize == 0 /*|| *(pendexpansionce + pos) < endexpansion*/) { /* the commented part is actually both redundant and dangerous */
       *(pendexpansionce + pos + 1) = endexpansion;
       *(pexpansionsize + pos + 1)  = expansionsize;
     }
@@ -440,7 +548,7 @@ int uprv_uca_setMaxExpansion(uint32_t           endexpansion,
 * @param status error status
 * @returns size of the maxexpansion and maxsize used.
 */
-int uprv_uca_setMaxJamoExpansion(UChar                  ch,
+static int uprv_uca_setMaxJamoExpansion(UChar                  ch,
                                  uint32_t               endexpansion,
                                  uint8_t                expansionsize,
                                  MaxJamoExpansionTable *maxexpansion,
@@ -475,9 +583,19 @@ int uprv_uca_setMaxJamoExpansion(UChar                  ch,
     /* we'll always make the first element 0, for easier manipulation */
     maxexpansion->endExpansionCE = 
                (uint32_t *)uprv_malloc(INIT_EXP_TABLE_SIZE * sizeof(uint32_t));
+    /* test for NULL */;
+    if (maxexpansion->endExpansionCE == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
     *(maxexpansion->endExpansionCE) = 0;
     maxexpansion->isV = 
                  (UBool *)uprv_malloc(INIT_EXP_TABLE_SIZE * sizeof(UBool));
+    /* test for NULL */;
+    if (maxexpansion->isV == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
     *(maxexpansion->isV) = 0;
     maxexpansion->size     = INIT_EXP_TABLE_SIZE;
     maxexpansion->position = 0;
@@ -513,7 +631,7 @@ int uprv_uca_setMaxJamoExpansion(UChar                  ch,
   *(pendexpansionce + maxexpansion->position) = endexpansion;
   *(maxexpansion->isV + maxexpansion->position) = isV;
   maxexpansion->position ++;
-  
+
   return maxexpansion->position;
 }
 
@@ -587,7 +705,7 @@ static void uprv_uca_unsafeCPAddCCNZ(tempUCATable *t, UErrorCode *status) {
     }
 }
 
-uint32_t uprv_uca_addPrefix(tempUCATable *t, uint32_t CE, 
+static uint32_t uprv_uca_addPrefix(tempUCATable *t, uint32_t CE, 
                                  UCAElements *element, UErrorCode *status) {
   // currently the longest prefix we're supporting in Japanese is two characters
   // long. Although this table could quite easily mimic complete contraction stuff
@@ -623,21 +741,6 @@ uint32_t uprv_uca_addPrefix(tempUCATable *t, uint32_t CE,
       }
     }
 
-#if 0
-    // These are all fairly small prefixes... We should be fine...
-    UChar nfcBuffer[256];
-    uint32_t nfcSize = unorm_normalize(element->prefix, element->prefixSize, UNORM_NFC, 0, nfcBuffer, 256, status);
-    for (j = 1; j<nfcSize; j++) {   /* First add NFC prefix chars to unsafe CP hash table */
-      // Unless it is a trail surrogate, which is handled algoritmically and 
-      // shouldn't take up space in the table.
-      if(!(UTF_IS_TRAIL(nfcBuffer[j]))) {
-        unsafeCPSet(t->unsafeCP, nfcBuffer[j]);
-      }
-    }
-
-    element->prefixSize = nfcSize;
-#endif
-
     UChar tempPrefix = 0;
 
     for(j = 0; j < /*nfcSize*/element->prefixSize/2; j++) { // prefixes are going to be looked up backwards
@@ -722,7 +825,7 @@ uint32_t uprv_uca_addPrefix(tempUCATable *t, uint32_t CE,
 // in the contraction, it is going to be handled as a pair of code units,
 // as it doesn't affect the performance AND handling surrogates specially
 // would complicate code way too much.
-uint32_t uprv_uca_addContraction(tempUCATable *t, uint32_t CE, 
+static uint32_t uprv_uca_addContraction(tempUCATable *t, uint32_t CE, 
                                  UCAElements *element, UErrorCode *status) {
     CntTable *contractions = t->contractions;
     UChar32 cp;
@@ -849,6 +952,17 @@ static uint32_t uprv_uca_processContraction(CntTable *contractions, UCAElements
 
 static uint32_t uprv_uca_finalizeAddition(tempUCATable *t, UCAElements *element, UErrorCode *status) {
   uint32_t CE = UCOL_NOT_FOUND;
+  // This should add a completely ignorable element to the 
+  // unsafe table, so that backward iteration will skip
+  // over it when treating contractions.
+  uint32_t i = 0;
+  if(element->mapCE == 0) {
+    for(i = 0; i < element->cSize; i++) {
+      if(!UTF_IS_TRAIL(element->cPoints[i])) {
+        unsafeCPSet(t->unsafeCP, element->cPoints[i]);
+      }
+    }
+  }
   if(element->cSize > 1) { /* we're adding a contraction */
     uint32_t i = 0;
     UChar32 cp;
@@ -898,15 +1012,11 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
   if(U_FAILURE(*status)) {
       return 0xFFFF;
   }
+
+  element->mapCE = 0; // clear mapCE so that we can catch expansions
+
   if(element->noOfCEs == 1) {
-    if(element->isThai == FALSE) {
-      element->mapCE = element->CEs[0];
-    } else { /* add thai - totally bad here */
-      expansion = (uint32_t)(UCOL_SPECIAL_FLAG | (THAI_TAG<<UCOL_TAG_SHIFT) 
-        | ((uprv_uca_addExpansion(expansions, element->CEs[0], status)+(headersize>>2))<<4) 
-        | 0x1);
-      element->mapCE = expansion;
-    }
+    element->mapCE = element->CEs[0];      
   } else {     
     /* ICU 2.1 long primaries */
     /* unfortunately, it looks like we have to look for a long primary here */
@@ -937,7 +1047,7 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
       expansion = (uint32_t)(UCOL_SPECIAL_FLAG | (EXPANSION_TAG<<UCOL_TAG_SHIFT) 
         | ((uprv_uca_addExpansion(expansions, element->CEs[0], status)+(headersize>>2))<<4)
         & 0xFFFFF0);
-
+        
       for(i = 1; i<element->noOfCEs; i++) {
         uprv_uca_addExpansion(expansions, element->CEs[i], status);
       }
@@ -962,6 +1072,37 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
     }
   }
 
+  // We treat digits differently - they are "uber special" and should be
+  // processed differently if numeric collation is on. 
+  UChar32 uniChar = 0;
+  //printElement(element);
+  if ((element->cSize == 2) && U16_IS_LEAD(element->uchars[0])){
+      uniChar = U16_GET_SUPPLEMENTARY(element->uchars[0], element->uchars[1]);
+  } else if (element->cSize == 1){
+      uniChar = element->uchars[0];
+  }
+
+  // Here, we either have one normal CE OR mapCE is set. Therefore, we stuff only
+  // one element to the expansion buffer. When we encounter a digit and we don't 
+  // do numeric collation, we will just pick the CE we have and break out of case
+  // (see ucol.cpp ucol_prv_getSpecialCE && ucol_prv_getSpecialPrevCE). If we picked
+  // a special, further processing will occur. If it's a simple CE, we'll return due
+  // to how the loop is constructed.
+  if (uniChar != 0 && u_isdigit(uniChar)){
+      expansion = (uint32_t)(UCOL_SPECIAL_FLAG | (DIGIT_TAG<<UCOL_TAG_SHIFT) | 1); // prepare the element
+      if(element->mapCE) { // if there is an expansion, we'll pick it here
+        expansion |= ((uprv_uca_addExpansion(expansions, element->mapCE, status)+(headersize>>2))<<4);
+      } else {
+        expansion |= ((uprv_uca_addExpansion(expansions, element->CEs[0], status)+(headersize>>2))<<4);
+      }
+      element->mapCE = expansion;
+      
+      // Need to go back to the beginning of the digit string if in the middle!
+      if(uniChar <= 0xFFFF) { // supplementaries are always unsafe. API takes UChars
+        unsafeCPSet(t->unsafeCP, (UChar)uniChar);
+      }
+  }
+
   // here we want to add the prefix structure.
   // I will try to process it as a reverse contraction, if possible.
   // prefix buffer is already reversed.
@@ -973,6 +1114,11 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
     // for canonical closure.
 
     UCAElements *composed = (UCAElements *)uprv_malloc(sizeof(UCAElements));
+    /* test for NULL */
+    if (composed == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return 0;
+    }
     uprv_memcpy(composed, element, sizeof(UCAElements));
     composed->cPoints = composed->uchars;
     composed->prefix = composed->prefixChars;
@@ -987,6 +1133,11 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
       } else { // no code points, so this spot is clean
         element->mapCE = uprv_uca_addPrefix(t, UCOL_NOT_FOUND, element, status);
         uCE = (UCAElements *)uprv_malloc(sizeof(UCAElements));
+        /* test for NULL */
+        if (uCE == NULL) {
+            *status = U_MEMORY_ALLOCATION_ERROR;
+            return 0;
+        }
         uprv_memcpy(uCE, element, sizeof(UCAElements));
         uCE->cPoints = uCE->uchars;
         uhash_put(t->prefixLookup, uCE, uCE, status);
@@ -1006,31 +1157,14 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
     UnicodeString source(element->cPoints, element->cSize);
     CanonicalIterator it(source, *status);
     source = it.next();
-    while(source.length() > 0) {
+    while(!source.isBogus()) {
       if(Normalizer::quickCheck(source, UNORM_FCD, *status) != UNORM_NO) {
         element->cSize = source.extract(element->cPoints, 128, *status);
         uprv_uca_finalizeAddition(t, element, status);
       }
       source = it.next();
     }
-#if 0
-    CE = uprv_uca_finalizeAddition(t, element, status);  
-    UChar composed[256];
-    uint32_t compLen = unorm_normalize(element->cPoints, element->cSize, UNORM_NFC, 0, composed, 256, status);;
-
-    if(compLen != element->cSize || uprv_memcmp(composed, element->cPoints, element->cSize*sizeof(UChar))) {
-      // composed form of a contraction is different than the decomposed form!
-      // do it!
-#ifdef UCOL_DEBUG
-      fprintf(stderr, "Adding composed for %04X->%04X\n", *element->cPoints, *composed);
-#endif
-      element->cSize = compLen;
-      uprv_memcpy(element->cPoints, composed, element->cSize*sizeof(UChar));
-      uprv_uca_finalizeAddition(t, element, status);
-    }
-#else
     CE = element->mapCE;
-#endif
   } else {
       CE = uprv_uca_finalizeAddition(t, element, status);  
   }
@@ -1040,7 +1174,7 @@ uprv_uca_addAnElement(tempUCATable *t, UCAElements *element, UErrorCode *status)
 
 
 /*void uprv_uca_getMaxExpansionJamo(CompactEIntArray       *mapping, */
-void uprv_uca_getMaxExpansionJamo(UNewTrie       *mapping, 
+static void uprv_uca_getMaxExpansionJamo(UNewTrie       *mapping, 
                                   MaxExpansionTable     *maxexpansion,
                                   MaxJamoExpansionTable *maxjamoexpansion,
                                   UBool                  jamospecial,
@@ -1050,7 +1184,7 @@ void uprv_uca_getMaxExpansionJamo(UNewTrie       *mapping,
   const uint32_t TBASE  = 0x11A8;
   const uint32_t VCOUNT = 21;
   const uint32_t TCOUNT = 28;
-  
+
   uint32_t v = VBASE + VCOUNT - 1;
   uint32_t t = TBASE + TCOUNT - 1;
   uint32_t ce;
@@ -1114,7 +1248,11 @@ getFoldedValue(UNewTrie *trie, UChar32 start, int32_t offset)
       tag = getCETag(value);
       if(inBlockZero == TRUE) {
           start+=UTRIE_DATA_BLOCK_LENGTH;
-      } else if(value!=0 && tag != IMPLICIT_TAG && tag != NOT_FOUND_TAG) {
+      } else if(!(isSpecial(value) && (tag == IMPLICIT_TAG || tag == NOT_FOUND_TAG))) {
+        /* These are values that are starting in either UCA (IMPLICIT_TAG) or in the 
+         * tailorings (NOT_FOUND_TAG). Presence of these tags means that there is 
+         * nothing in this position and that it should be skipped.
+         */
 #ifdef UCOL_DEBUG
         static int32_t count = 1;
         fprintf(stdout, "%i, Folded %08X, value %08X\n", count++, start, value);
@@ -1199,22 +1337,27 @@ uprv_uca_assembleTable(tempUCATable *t, UErrorCode *status) {
 
 
     dataStart = (uint8_t *)uprv_malloc(toAllocate);
+    /* test for NULL */
+    if (dataStart == NULL) {
+        *status = U_MEMORY_ALLOCATION_ERROR;
+        return NULL;
+    }
 
     UCATableHeader *myData = (UCATableHeader *)dataStart;
-    uprv_memcpy(myData, t->image, sizeof(UCATableHeader));
+    // Please, do reset all the fields!
+    uprv_memset(dataStart, 0, toAllocate);
+    // Make sure we know this is reset
+    myData->magic = UCOL_HEADER_MAGIC;
+    myData->isBigEndian = U_IS_BIG_ENDIAN;
+    myData->charSetFamily = U_CHARSET_FAMILY;
+    myData->formatVersion[0] = UCA_FORMAT_VERSION_0;
+    myData->formatVersion[1] = UCA_FORMAT_VERSION_1;
+    myData->formatVersion[2] = UCA_FORMAT_VERSION_2;
+    myData->formatVersion[3] = UCA_FORMAT_VERSION_3;
+    myData->jamoSpecial = t->image->jamoSpecial;
 
-#if 0
-    /* above memcpy should save us from problems */
-    myData->variableTopValue = t->image->variableTopValue;
-    myData->strength = t->image->strength;
-    myData->frenchCollation = t->image->frenchCollation;
-    myData->alternateHandling = t->image->alternateHandling; /* attribute for handling variable elements*/
-    myData->caseFirst = t->image->caseFirst;         /* who goes first, lower case or uppercase */
-    myData->caseLevel = t->image->caseLevel;         /* do we have an extra case level */
-    myData->normalizationMode = t->image->normalizationMode; /* attribute for normalization */
-    myData->version[0] = t->image->version[0];
-    myData->version[1] = t->image->version[1];  
-#endif
+    // Don't copy stuff from UCA header!
+    //uprv_memcpy(myData, t->image, sizeof(UCATableHeader));
 
     myData->contractionSize = contractionsSize;
 
@@ -1227,7 +1370,7 @@ uprv_uca_assembleTable(tempUCATable *t, UErrorCode *status) {
     /* copy expansions */
     /*myData->expansion = (uint32_t *)dataStart+tableOffset;*/
     myData->expansion = tableOffset;
-    memcpy(dataStart+tableOffset, expansions->CEs, expansions->position*sizeof(uint32_t));
+    uprv_memcpy(dataStart+tableOffset, expansions->CEs, expansions->position*sizeof(uint32_t));
     tableOffset += (uint32_t)(paddedsize(expansions->position*sizeof(uint32_t)));
 
     /* contractions block */
@@ -1235,23 +1378,23 @@ uprv_uca_assembleTable(tempUCATable *t, UErrorCode *status) {
       /* copy contraction index */
       /*myData->contractionIndex = (UChar *)(dataStart+tableOffset);*/
       myData->contractionIndex = tableOffset;
-      memcpy(dataStart+tableOffset, contractions->codePoints, contractionsSize*sizeof(UChar));
+      uprv_memcpy(dataStart+tableOffset, contractions->codePoints, contractionsSize*sizeof(UChar));
       tableOffset += (uint32_t)(paddedsize(contractionsSize*sizeof(UChar)));
 
       /* copy contraction collation elements */
       /*myData->contractionCEs = (uint32_t *)(dataStart+tableOffset);*/
       myData->contractionCEs = tableOffset;
-      memcpy(dataStart+tableOffset, contractions->CEs, contractionsSize*sizeof(uint32_t));
+      uprv_memcpy(dataStart+tableOffset, contractions->CEs, contractionsSize*sizeof(uint32_t));
       tableOffset += (uint32_t)(paddedsize(contractionsSize*sizeof(uint32_t)));
     } else {
       myData->contractionIndex = 0;
-      myData->contractionIndex = 0;
+      myData->contractionCEs = 0;
     }
 
     /* copy mapping table */
     /*myData->mappingPosition = dataStart+tableOffset;*/
     /*myData->mappingPosition = tableOffset;*/
-    /*memcpy(dataStart+tableOffset, flattened, mappingSize);*/
+    /*uprv_memcpy(dataStart+tableOffset, flattened, mappingSize);*/
 
     myData->mappingPosition = tableOffset;
     utrie_serialize(mapping, dataStart+tableOffset, toAllocate-tableOffset, getFoldedValue, FALSE, status);
@@ -1259,39 +1402,31 @@ uprv_uca_assembleTable(tempUCATable *t, UErrorCode *status) {
     // This is debug code to dump the contents of the trie. It needs two functions defined above
     {
       UTrie UCAt = { 0 };
+      uint32_t trieWord;
       utrie_unserialize(&UCAt, dataStart+tableOffset, 9999999, status);
       UCAt.getFoldingOffset = myGetFoldingOffset;
       if(U_SUCCESS(*status)) {
         utrie_enum(&UCAt, NULL, enumRange, NULL);
       }
+      trieWord = UTRIE_GET32_FROM_LEAD(UCAt, 0xDC01) 
     }
 #endif
     tableOffset += paddedsize(mappingSize);
 
 
     int32_t i = 0;
-#if 0
-    /* construct the fast tracker for latin one*/
-    myData->latinOneMapping = tableOffset;
-    uint32_t *store = (uint32_t*)(dataStart+tableOffset);
-    for(i = 0; i<=0xFF; i++) {
-        /**(store++) = ucmpe32_get(mapping,i);*/
-        *(store++) = utrie_get32(mapping, i, NULL);
-        tableOffset+=(uint32_t)(sizeof(uint32_t));
-    }
-#endif
 
     /* copy max expansion table */
     myData->endExpansionCE      = tableOffset;
-    myData->endExpansionCECount = maxexpansion->position;
+    myData->endExpansionCECount = maxexpansion->position - 1;
     /* not copying the first element which is a dummy */
     uprv_memcpy(dataStart + tableOffset, maxexpansion->endExpansionCE + 1, 
-                maxexpansion->position * sizeof(uint32_t));
-    tableOffset += (uint32_t)(paddedsize(maxexpansion->position * sizeof(uint32_t)));
+                (maxexpansion->position - 1) * sizeof(uint32_t));
+    tableOffset += (uint32_t)(paddedsize((maxexpansion->position)* sizeof(uint32_t)));
     myData->expansionCESize = tableOffset;
     uprv_memcpy(dataStart + tableOffset, maxexpansion->expansionCESize + 1, 
-                maxexpansion->position * sizeof(uint8_t));
-    tableOffset += (uint32_t)(paddedsize(maxexpansion->position * sizeof(uint8_t)));
+                (maxexpansion->position - 1) * sizeof(uint8_t));
+    tableOffset += (uint32_t)(paddedsize((maxexpansion->position)* sizeof(uint8_t)));
 
     /* Unsafe chars table.  Finish it off, then copy it. */
     uprv_uca_unsafeCPAddCCNZ(t, status);
@@ -1331,4 +1466,127 @@ uprv_uca_assembleTable(tempUCATable *t, UErrorCode *status) {
     return myData;
 }
 
-U_NAMESPACE_END
+
+struct enumStruct {
+  tempUCATable *t;
+  UCollator *tempColl;
+  UCollationElements* colEl;
+  int32_t noOfClosures;
+  UErrorCode *status;
+};
+U_CDECL_BEGIN
+static UBool U_CALLCONV
+_enumCategoryRangeClosureCategory(const void *context, UChar32 start, UChar32 limit, UCharCategory type) {
+
+  UErrorCode *status = ((enumStruct *)context)->status;
+  tempUCATable *t = ((enumStruct *)context)->t;
+  UCollator *tempColl = ((enumStruct *)context)->tempColl;
+  UCollationElements* colEl = ((enumStruct *)context)->colEl;
+  UCAElements el;
+  UChar decomp[256] = { 0 };
+  int32_t noOfDec = 0;
+
+  UChar32 u32 = 0;
+  UChar comp[2];
+  uint32_t len = 0;
+
+  if (type != U_UNASSIGNED && type != U_PRIVATE_USE_CHAR) { // if the range is assigned - we might ommit more categories later
+    for(u32 = start; u32 < limit; u32++) {
+      noOfDec = unorm_getDecomposition(u32, FALSE, decomp, 256);
+      //if((noOfDec = unorm_normalize(comp, len, UNORM_NFD, 0, decomp, 256, status)) > 1
+        //|| (noOfDec == 1 && *decomp != (UChar)u32))
+      if(noOfDec > 0) // if we're positive, that means there is no decomposition
+      {
+        len = 0;
+        UTF_APPEND_CHAR_UNSAFE(comp, len, u32);
+        if(ucol_strcoll(tempColl, comp, len, decomp, noOfDec) != UCOL_EQUAL) {
+#ifdef UCOL_DEBUG
+          fprintf(stderr, "Closure: %08X -> ", u32);
+          uint32_t i = 0;
+          for(i = 0; i<noOfDec; i++) {
+            fprintf(stderr, "%04X ", decomp[i]);
+          }
+          fprintf(stderr, "\n");
+#endif
+          ((enumStruct *)context)->noOfClosures++;
+          el.cPoints = decomp;
+          el.cSize = noOfDec;
+          el.noOfCEs = 0;
+          el.prefix = el.prefixChars;
+          el.prefixSize = 0;
+
+          UCAElements *prefix=(UCAElements *)uhash_get(t->prefixLookup, &el);
+          if(prefix == NULL) {
+            el.cPoints = comp;
+            el.cSize = len;
+            el.prefix = el.prefixChars;
+            el.prefixSize = 0;
+            el.noOfCEs = 0;
+            ucol_setText(colEl, decomp, noOfDec, status);
+            while((el.CEs[el.noOfCEs] = ucol_next(colEl, status)) != (uint32_t)UCOL_NULLORDER) {
+              el.noOfCEs++;
+            }
+          } else {
+            el.cPoints = comp;
+            el.cSize = len;
+            el.prefix = el.prefixChars;
+            el.prefixSize = 0;
+            el.noOfCEs = 1;
+            el.CEs[0] = prefix->mapCE;
+            // This character uses a prefix. We have to add it 
+            // to the unsafe table, as it decomposed form is already
+            // in. In Japanese, this happens for \u309e & \u30fe
+            // Since unsafeCPSet is static in ucol_elm, we are going
+            // to wrap it up in the uprv_uca_unsafeCPAddCCNZ function
+          }
+          uprv_uca_addAnElement(t, &el, status);
+        }
+      }
+    }
+  }
+  return TRUE;
+}
+U_CDECL_END
+
+U_CAPI int32_t U_EXPORT2
+uprv_uca_canonicalClosure(tempUCATable *t, UErrorCode *status) 
+{
+  enumStruct context;
+  context.noOfClosures = 0;
+  if(U_SUCCESS(*status)) {
+    UCollator *tempColl = NULL;
+    tempUCATable *tempTable = uprv_uca_cloneTempTable(t, status);
+
+    UCATableHeader *tempData = uprv_uca_assembleTable(tempTable, status);
+    tempColl = ucol_initCollator(tempData, 0, t->UCA, status);
+    uprv_uca_closeTempTable(tempTable);    
+
+    if(U_SUCCESS(*status)) {
+      tempColl->rb = NULL;
+      tempColl->elements = NULL;
+      tempColl->validLocale = NULL;
+      tempColl->requestedLocale = NULL;
+      tempColl->hasRealData = TRUE;
+      tempColl->freeImageOnClose = TRUE;
+    } else if(tempData != 0) {
+      uprv_free(tempData);
+    }
+
+    /* produce canonical closure */
+    UCollationElements* colEl = ucol_openElements(tempColl, NULL, 0, status);
+
+    context.t = t;
+    context.tempColl = tempColl;
+    context.colEl = colEl;
+    context.status = status;
+    u_enumCharTypes(_enumCategoryRangeClosureCategory, &context);
+
+    ucol_closeElements(colEl);
+    ucol_close(tempColl);
+  }
+  return context.noOfClosures;
+}
+
+#endif /* #if !UCONFIG_NO_COLLATION */
+
+
