@@ -1,6 +1,6 @@
 /*  
 ********************************************************************************
-*   Copyright (C) 1997-2001, International Business Machines
+*   Copyright (C) 1997-2005, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 ********************************************************************************
 *
@@ -19,8 +19,17 @@
 #define DTFMTSYM_H
  
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
+#include "unicode/uobject.h"
 #include "unicode/locid.h"
-#include "unicode/resbund.h"
+#include "unicode/ures.h"
+
+/**
+ * \file 
+ * \brief C++ API: Symbols for formatting dates.
+ */
 
 U_NAMESPACE_BEGIN
 
@@ -54,74 +63,124 @@ class SimpleDateFormat;
  * they feel easy to remember. Or they can change the representative cities
  * originally picked by default to using their favorite ones.
  * <P>
- * New DateFormatSymbols sub-classes may be added to support SimpleDateFormat
- * for date-time formatting for additional locales.
+ * DateFormatSymbols are not expected to be subclassed. Data for a calendar is
+ * loaded out of resource bundles.  The 'type' parameter indicates the type of
+ * calendar, for example, "gregorian" or "japanese".  If the type is not gregorian
+ * (or NULL, or an empty string) then the type is appended to the resource name,
+ * for example,  'Eras_japanese' instead of 'Eras'.   If the resource 'Eras_japanese' did
+ * not exist (even in root), then this class will fall back to just 'Eras', that is,
+ * Gregorian data.  Therefore, the calendar implementor MUST ensure that the root
+ * locale at least contains any resources that are to be particularized for the
+ * calendar type.
  */
-class U_I18N_API DateFormatSymbols {
+class U_I18N_API DateFormatSymbols : public UObject {
 public:
     /**
      * Construct a DateFormatSymbols object by loading format data from
-     * resources for the default locale.
+     * resources for the default locale, in the default calendar (Gregorian).
      * <P>
      * NOTE: This constructor will never fail; if it cannot get resource
      * data for the default locale, it will return a last-resort object
      * based on hard-coded strings.
      *
-     * @param status    Output param set to success of failure.  Failure
+     * @param status    Status code.  Failure
      *                  results if the resources for the default cannot be
      *                  found or cannot be loaded
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormatSymbols(UErrorCode& status);
 
     /**
      * Construct a DateFormatSymbols object by loading format data from
-     * resources for the given locale.
+     * resources for the given locale, in the default calendar (Gregorian).
      *
      * @param locale    Locale to load format data from.
-     * @param status    Output param set to success of failure.  Failure
+     * @param status    Status code.  Failure
      *                  results if the resources for the locale cannot be
      *                  found or cannot be loaded
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormatSymbols(const Locale& locale,
                       UErrorCode& status);
 
+    /**
+     * Construct a DateFormatSymbols object by loading format data from
+     * resources for the default locale, in the default calendar (Gregorian).
+     * <P>
+     * NOTE: This constructor will never fail; if it cannot get resource
+     * data for the default locale, it will return a last-resort object
+     * based on hard-coded strings.
+     *
+     * @param type      Type of calendar (as returned by Calendar::getType). 
+     *                  Will be used to access the correct set of strings.
+     *                  (NULL or empty string defaults to "gregorian".)
+     * @param status    Status code.  Failure
+     *                  results if the resources for the default cannot be
+     *                  found or cannot be loaded
+     * @internal
+     */
+    DateFormatSymbols(const char *type, UErrorCode& status);
+
+    /**
+     * Construct a DateFormatSymbols object by loading format data from
+     * resources for the given locale, in the default calendar (Gregorian).
+     *
+     * @param locale    Locale to load format data from.
+     * @param type      Type of calendar (as returned by Calendar::getType). 
+     *                  Will be used to access the correct set of strings.
+     *                  (NULL or empty string defaults to "gregorian".)
+     * @param status    Status code.  Failure
+     *                  results if the resources for the locale cannot be
+     *                  found or cannot be loaded
+     * @internal
+     */
+    DateFormatSymbols(const Locale& locale,
+                      const char *type,
+                      UErrorCode& status);
+
     /**
      * Copy constructor.
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormatSymbols(const DateFormatSymbols&);
 
     /**
      * Assignment operator.
-     * @stable
+     * @stable ICU 2.0
      */
     DateFormatSymbols& operator=(const DateFormatSymbols&);
 
     /**
      * Destructor. This is nonvirtual because this class is not designed to be
      * subclassed.
-     * @stable
+     * @stable ICU 2.0
      */
-    ~DateFormatSymbols();
+    virtual ~DateFormatSymbols();
 
     /**
      * Return true if another object is semantically equal to this one.
-     * @stable
+     *
+     * @param other    the DateFormatSymbols object to be compared with.
+     * @return         true if other is semantically equal to this.
+     * @stable ICU 2.0
      */
     UBool operator==(const DateFormatSymbols& other) const;
 
     /**
      * Return true if another object is semantically unequal to this one.
-     * @stable
+     *
+     * @param other    the DateFormatSymbols object to be compared with.
+     * @return         true if other is semantically unequal to this.
+     * @stable ICU 2.0
      */
     UBool operator!=(const DateFormatSymbols& other) const { return !operator==(other); }
 
     /**
      * Gets era strings. For example: "AD" and "BC".
-     * @return the era strings.
-     * @stable
+     *
+     * @param count    Filled in with length of the array.
+     * @return         the era strings.
+     * @stable ICU 2.0
      */
     const UnicodeString* getEras(int32_t& count) const;
 
@@ -129,78 +188,150 @@ public:
      * Sets era strings. For example: "AD" and "BC".
      * @param eras  Array of era strings (DateFormatSymbols retains ownership.)
      * @param count Filled in with length of the array.
-     * @stable
+     * @stable ICU 2.0
      */
     void setEras(const UnicodeString* eras, int32_t count);
 
+    /**
+     * Gets era name strings. For example: "Anno Domini" and "Before Christ".
+     *
+     * @param count    Filled in with length of the array.
+     * @return         the era name strings.
+     * @draft ICU 3.4
+     */
+    const UnicodeString* getEraNames(int32_t& count) const;
+
     /**
      * Gets month strings. For example: "January", "February", etc.
      * @param count Filled in with length of the array.
      * @return the month strings. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @stable ICU 2.0
      */
     const UnicodeString* getMonths(int32_t& count) const;
 
     /**
      * Sets month strings. For example: "January", "February", etc.
-     * @param newMonths the new month strings. (not adopted; caller retains ownership)
-     * @stable
+     *
+     * @param months    the new month strings. (not adopted; caller retains ownership)
+     * @param count     Filled in with length of the array.
+     * @stable ICU 2.0
      */
     void setMonths(const UnicodeString* months, int32_t count);
 
     /**
      * Gets short month strings. For example: "Jan", "Feb", etc.
+     *
+     * @param count Filled in with length of the array.
      * @return the short month strings. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @stable ICU 2.0
      */
     const UnicodeString* getShortMonths(int32_t& count) const;
 
     /**
      * Sets short month strings. For example: "Jan", "Feb", etc.
-     * @param newShortMonths the new short month strings. (not adopted; caller retains ownership)
-     * @stable
+     * @param count        Filled in with length of the array.
+     * @param shortMonths  the new short month strings. (not adopted; caller retains ownership)
+     * @stable ICU 2.0
      */
     void setShortMonths(const UnicodeString* shortMonths, int32_t count);
 
+    enum DtContextType {
+        /**
+         * Selector for format date context
+         * @draft ICU 3.4
+         */
+         FORMAT = 0,
+       /**
+         * Selector for standalone date context
+         * @draft ICU 3.4
+         */
+         STANDALONE = 1
+    };
+
+    enum DtWidthType {
+        /**
+         * Selector for wide strings
+         * @draft ICU 3.4
+         */
+         WIDE = 4,
+       /**
+         * Selector for abbreviated strings
+         * @draft ICU 3.4
+         */
+         ABBREVIATED = 3,
+       /**
+         * Selector for narrow strings
+         * @draft ICU 3.4
+         */
+         NARROW = 5
+    };
+
+    /**
+     * Gets month strings by width and context. For example: "January", "February", etc.
+     * @param count Filled in with length of the array.
+     * @param context The day formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
+     * @return the month strings. (DateFormatSymbols retains ownership.)
+     * @draft ICU 3.4
+     */
+    const UnicodeString* getMonths(int32_t& count, DtContextType context, DtWidthType width) const;
+
     /**
      * Gets weekday strings. For example: "Sunday", "Monday", etc.
+     * @param count        Filled in with length of the array.
      * @return the weekday strings. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @stable ICU 2.0
      */
     const UnicodeString* getWeekdays(int32_t& count) const;
 
+
     /**
      * Sets weekday strings. For example: "Sunday", "Monday", etc.
-     * @param newWeekdays the new weekday strings. (not adopted; caller retains ownership)
-     * @stable
+     * @param weekdays     the new weekday strings. (not adopted; caller retains ownership)
+     * @param count        Filled in with length of the array.
+     * @stable ICU 2.0
      */
     void setWeekdays(const UnicodeString* weekdays, int32_t count);
 
     /**
      * Gets short weekday strings. For example: "Sun", "Mon", etc.
-     * @return the short weekday strings. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @param count        Filled in with length of the array.
+     * @return             the short weekday strings. (DateFormatSymbols retains ownership.)
+     * @stable ICU 2.0
      */
     const UnicodeString* getShortWeekdays(int32_t& count) const;
 
     /**
      * Sets short weekday strings. For example: "Sun", "Mon", etc.
-     * @param newShortWeekdays the new short weekday strings. (not adopted; caller retains ownership)
-     * @stable
+     * @param shortWeekdays  the new short weekday strings. (not adopted; caller retains ownership)
+     * @param count          Filled in with length of the array.
+     * @stable ICU 2.0
      */
     void setShortWeekdays(const UnicodeString* shortWeekdays, int32_t count);
 
+    /**
+     * Gets weekday strings by width and context. For example: "Sunday", "Monday", etc.
+     * @param count   Filled in with length of the array.
+     * @param context The day formatting context, either FORMAT or STANDALONE
+     * @param width   The width of returned strings, either WIDE, ABBREVIATED, or NARROW.
+     * @return the month strings. (DateFormatSymbols retains ownership.)
+     * @draft ICU 3.4
+     */
+    const UnicodeString* getWeekdays(int32_t& count, DtContextType context, DtWidthType width) const;
+
     /**
      * Gets AM/PM strings. For example: "AM" and "PM".
-     * @return the weekday strings. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @param count        Filled in with length of the array.
+     * @return             the weekday strings. (DateFormatSymbols retains ownership.)
+     * @stable ICU 2.0
      */
     const UnicodeString* getAmPmStrings(int32_t& count) const;
 
     /**
      * Sets ampm strings. For example: "AM" and "PM".
-     * @param newAmpms the new ampm strings. (not adopted; caller retains ownership)
-     * @stable
+     * @param ampms        the new ampm strings. (not adopted; caller retains ownership)
+     * @param count        Filled in with length of the array.
+     * @stable ICU 2.0
      */
     void setAmPmStrings(const UnicodeString* ampms, int32_t count);
 
@@ -209,7 +340,7 @@ public:
      * @param rowCount      Output param to receive number of rows.
      * @param columnCount   Output param to receive number of columns.
      * @return              The timezone strings as a 2-d array. (DateFormatSymbols retains ownership.)
-     * @stable
+     * @stable ICU 2.0
      */
     const UnicodeString** getZoneStrings(int32_t& rowCount, int32_t& columnCount) const;
 
@@ -218,26 +349,22 @@ public:
      * @param strings       The timezone strings as a 2-d array to be copied. (not adopted; caller retains ownership)
      * @param rowCount      The number of rows (count of first index).
      * @param columnCount   The number of columns (count of second index).
-     * @stable
+     * @stable ICU 2.0
      */
     void setZoneStrings(const UnicodeString* const* strings, int32_t rowCount, int32_t columnCount);
 
     /**
      * Get the non-localized date-time pattern characters.
-     * @deprecated remove after Aug 2002. Use getPatternUChars instead.
+     * @return    the non-localized date-time pattern characters
+     * @stable ICU 2.0
      */
-    static const UnicodeString& getPatternChars(void);
-
-    /**
-     * Get the non-localized date-time pattern characters.
-     * @stable
-     */
-    static const UChar *getPatternUChars(void);
+    static const UChar * U_EXPORT2 getPatternUChars(void);
 
     /**
      * Gets localized date-time pattern characters. For example: 'u', 't', etc.
-     * @return the localized date-time pattern characters.
-     * @stable
+     * @param result    Output param which will receive the localized date-time pattern characters.
+     * @return          A reference to 'result'.
+     * @stable ICU 2.0
      */
     UnicodeString& getLocalPatternChars(UnicodeString& result) const;
 
@@ -245,25 +372,35 @@ public:
      * Sets localized date-time pattern characters. For example: 'u', 't', etc.
      * @param newLocalPatternChars the new localized date-time
      * pattern characters.
-     * @stable
+     * @stable ICU 2.0
      */
     void setLocalPatternChars(const UnicodeString& newLocalPatternChars);
 
-private:
     /**
-     * Resource bundle file suffix and tag names used by this class.
+     * Returns the locale for this object. Two flavors are available:
+     * valid and actual locale.
+     * @draft ICU 2.8 likely to change after ICU 3.0, based on feedback
      */
-    static const char fgErasTag[];   // resource bundle tag for era names
-    static const char fgMonthNamesTag[]; // resource bundle tag for month names
-    static const char fgMonthAbbreviationsTag[]; // resource bundle tag for month abbreviations
-    static const char fgDayNamesTag[];   // resource bundle tag for day names
-    static const char fgDayAbbreviationsTag[];   // resource bundle tag for day abbreviations
-    static const char fgAmPmMarkersTag[];    // resource bundle tag for AM/PM strings
+    Locale getLocale(ULocDataLocaleType type, UErrorCode& status) const;
 
-    static const char fgZoneStringsTag[];    // resource bundle tag for time zone names
-    static const char fgLocalPatternCharsTag[];  // resource bundle tag for localized pattern characters
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for the actual class.
+     *
+     * @stable ICU 2.2
+     */
+    virtual UClassID getDynamicClassID() const;
+
+    /**
+     * ICU "poor man's RTTI", returns a UClassID for this class.
+     *
+     * @stable ICU 2.2
+     */
+    static UClassID U_EXPORT2 getStaticClassID();
+
+private:
 
     friend class SimpleDateFormat;
+    friend class DateFormatSymbolsSingleSetter; // see udat.cpp
 
     /**
      * Era strings. For example: "AD" and "BC".
@@ -271,6 +408,12 @@ private:
     UnicodeString*  fEras;
     int32_t         fErasCount;
 
+    /**
+     * Era name strings. For example: "Anno Domini" and "Before Christ".
+     */
+    UnicodeString*  fEraNames;
+    int32_t         fEraNamesCount;
+
     /**
      * Month strings. For example: "January", "February", etc.
      */
@@ -283,6 +426,30 @@ private:
     UnicodeString*  fShortMonths;
     int32_t         fShortMonthsCount;
 
+    /**
+     * Narrow month strings. For example: "J", "F", etc.
+     */
+    UnicodeString*  fNarrowMonths;
+    int32_t         fNarrowMonthsCount;
+
+    /**
+     * Standalone Month strings. For example: "January", "February", etc.
+     */
+    UnicodeString*  fStandaloneMonths;
+    int32_t         fStandaloneMonthsCount;
+
+    /**
+     * Standalone Short month strings. For example: "Jan", "Feb", etc.
+     */
+    UnicodeString*  fStandaloneShortMonths;
+    int32_t         fStandaloneShortMonthsCount;
+
+    /**
+     * Standalone Narrow month strings. For example: "J", "F", etc.
+     */
+    UnicodeString*  fStandaloneNarrowMonths;
+    int32_t         fStandaloneNarrowMonthsCount;
+
     /**
      * Weekday strings. For example: "Sunday", "Monday", etc.
      */
@@ -295,6 +462,30 @@ private:
     UnicodeString*  fShortWeekdays;
     int32_t         fShortWeekdaysCount;
 
+    /**
+     * Narrow weekday strings. For example: "Sun", "Mon", etc.
+     */
+    UnicodeString*  fNarrowWeekdays;
+    int32_t         fNarrowWeekdaysCount;
+
+    /**
+     * Standalone Weekday strings. For example: "Sunday", "Monday", etc.
+     */
+    UnicodeString*  fStandaloneWeekdays;
+    int32_t         fStandaloneWeekdaysCount;
+
+    /**
+     * Standalone Short weekday strings. For example: "Sun", "Mon", etc.
+     */
+    UnicodeString*  fStandaloneShortWeekdays;
+    int32_t         fStandaloneShortWeekdaysCount;
+
+    /**
+     * Standalone Narrow weekday strings. For example: "Sun", "Mon", etc.
+     */
+    UnicodeString*  fStandaloneNarrowWeekdays;
+    int32_t         fStandaloneNarrowWeekdaysCount;
+
     /**
      * Ampm strings. For example: "AM" and "PM".
      */
@@ -314,35 +505,32 @@ private:
     UnicodeString   fLocalPatternChars;
 
 private:
+    /** valid/actual locale information 
+     *  these are always ICU locales, so the length should not be a problem
+     */
+    char validLocale[ULOC_FULLNAME_CAPACITY];
+    char actualLocale[ULOC_FULLNAME_CAPACITY];
 
-    /* Sizes for the last resort string arrays */
-    typedef enum LastResortSize {
-        kMonthNum = 13,
-        kMonthLen = 3,
-
-        kDayNum = 8,
-        kDayLen = 2,
-
-        kAmPmNum = 2,
-        kAmPmLen = 3,
-
-        kEraNum = 2,
-        kEraLen = 3,
-
-        kZoneNum = 5,
-        kZoneLen = 4
-    } LastResortSize;
-
-    void initField(UnicodeString **field, int32_t& length, const ResourceBundle data, UErrorCode &status);
-    void initField(UnicodeString **field, int32_t& length, const UChar *data, LastResortSize numStr, LastResortSize strLen, UErrorCode &status);
+    DateFormatSymbols(); // default constructor not implemented
 
     /**
      * Called by the constructors to actually load data from the resources
+     *
+     * @param locale               The locale to get symbols for.
+     * @param type                 Calendar Type (as from Calendar::getType())
+     * @param status               Input/output parameter, set to success or
+     *                             failure code upon return.
+     * @param useLastResortData    determine if use last resort data
      */
-    void initializeData(const Locale&, UErrorCode& status, UBool useLastResortData = FALSE);
+    void initializeData(const Locale&, const char *type, UErrorCode& status, UBool useLastResortData = FALSE);
 
     /**
      * Copy or alias an array in another object, as appropriate.
+     *
+     * @param dstArray    the copy destination array.
+     * @param dstCount    fill in with the lenth of 'dstArray'.
+     * @param srcArray    the source array to be copied.
+     * @param srcCount    the length of items to be copied from the 'srcArray'.
      */
     static void assignArray(UnicodeString*& dstArray,
                             int32_t& dstCount,
@@ -352,6 +540,12 @@ private:
     /**
      * Return true if the given arrays' contents are equal, or if the arrays are
      * identical (pointers are equal).
+     *
+     * @param array1   one array to be compared with.
+     * @param array2   another array to be compared with.
+     * @param count    the length of items to be copied.
+     * @return         true if the given arrays' contents are equal, or if the arrays are
+     *                 identical (pointers are equal).
      */
     static UBool arrayCompare(const UnicodeString* array1,
                              const UnicodeString* array2,
@@ -360,8 +554,7 @@ private:
     /**
      * Create a copy, in fZoneStrings, of the given zone strings array. The
      * member variables fZoneStringsRowCount and fZoneStringsColCount should be
-     * set already by the caller. The fIsOwned flags are not checked or set by
-     * this method; that is the caller's responsibility.
+     * set already by the caller.
      */
     void createZoneStrings(const UnicodeString *const * otherStrings);
 
@@ -381,24 +574,25 @@ private:
     int32_t _getZoneIndex(const UnicodeString& id) const;
 
     /**
-     * Delete all the storage owned by this object and reset the fIsOwned flag
-     * to indicate that arrays have been deleted.
+     * Delete all the storage owned by this object.
      */
     void dispose(void);
 
     /**
      * Copy all of the other's data to this.
+     * @param other the object to be copied.
      */
     void copyData(const DateFormatSymbols& other);
 
     /**
-     * Delete just the zone strings, if they are owned by this object. This
-     * method does NOT modify fIsOwned; the caller must handle that.
+     * Delete just the zone strings.
      */
     void disposeZoneStrings(void);
 };
 
 U_NAMESPACE_END
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif // _DTFMTSYM
 //eof
