@@ -1,6 +1,6 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2006, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
@@ -28,20 +28,27 @@
  * a customized collator object.  For example, number 1 to be sorted 
  * equlivalent to word 'one'. 
  */
-#ifndef INCLUDE_CALLCOLL_C
 
 #include <string.h>
 #include <stdlib.h>
+
 #include "unicode/utypes.h"
+
+#if !UCONFIG_NO_COLLATION
+
 #include "unicode/ucol.h"
 #include "unicode/uloc.h"
-#include "cstring.h"
+#include "unicode/ucoleitr.h"
+#include "unicode/ustring.h"
+#include "unicode/uclean.h"
+#include "unicode/putil.h"
+
 #include "cintltst.h"
 #include "ccolltst.h"
 #include "callcoll.h"
-#include "unicode/ustring.h"
+#include "calldata.h"
+#include "cstring.h"
 #include "cmemory.h"
-#include "unicode/ucoleitr.h"
 #include "ucol_imp.h"
 
 /* perform test with strength PRIMARY */
@@ -73,89 +80,9 @@ static void TestSurrogates(void);
 
 static void TestInvalidRules(void);
 
-#endif
+static void TestJitterbug1098(void);
 
-const UChar testSourceCases[][16] = {
-    {0x61, 0x62, 0x27, 0x63, 0},
-    {0x63, 0x6f, 0x2d, 0x6f, 0x70, 0},
-    {0x61, 0x62, 0},
-    {0x61, 0x6d, 0x70, 0x65, 0x72, 0x73, 0x61, 0x64, 0},
-    {0x61, 0x6c, 0x6c, 0},
-    {0x66, 0x6f, 0x75, 0x72, 0},
-    {0x66, 0x69, 0x76, 0x65, 0},
-    {0x31, 0},
-    {0x31, 0},
-    {0x31, 0},                                            /*  10 */
-    {0x32, 0},
-    {0x32, 0},
-    {0x48, 0x65, 0x6c, 0x6c, 0x6f, 0},
-    {0x61, 0x3c, 0x62, 0},
-    {0x61, 0x3c, 0x62, 0},
-    {0x61, 0x63, 0x63, 0},
-    {0x61, 0x63, 0x48, 0x63, 0},  /*  simple test */
-    {0x70, 0x00EA, 0x63, 0x68, 0x65, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x62, 0x63, 0},                                  /*  20 */
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x00E6, 0x63, 0},
-    {0x61, 0x63, 0x48, 0x63, 0},  /*  primary test */
-    {0x62, 0x6c, 0x61, 0x63, 0x6b, 0},
-    {0x66, 0x6f, 0x75, 0x72, 0},
-    {0x66, 0x69, 0x76, 0x65, 0},
-    {0x31, 0},
-    {0x61, 0x62, 0x63, 0},                                        /*  30 */
-    {0x61, 0x62, 0x63, 0},                                  
-    {0x61, 0x62, 0x63, 0x48, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x63, 0x48, 0x63, 0},                              /*  34 */
-    {0x61, 0x63, 0x65, 0x30},
-    {0x31, 0x30},
-    {0x70, 0x00EA,0x30}                                    /* 37     */
-};
-
-const UChar testTargetCases[][16] = {
-    {0x61, 0x62, 0x63, 0x27, 0},
-    {0x43, 0x4f, 0x4f, 0x50, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x26, 0},
-    {0x26, 0},
-    {0x34, 0},
-    {0x35, 0},
-    {0x6f, 0x6e, 0x65, 0},
-    {0x6e, 0x6e, 0x65, 0},
-    {0x70, 0x6e, 0x65, 0},                                  /*  10 */
-    {0x74, 0x77, 0x6f, 0},
-    {0x75, 0x77, 0x6f, 0},
-    {0x68, 0x65, 0x6c, 0x6c, 0x4f, 0},
-    {0x61, 0x3c, 0x3d, 0x62, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x43, 0x48, 0x63, 0},
-    {0x61, 0x43, 0x48, 0x63, 0},  /*  simple test */
-    {0x70, (UChar)0x00E9, 0x63, 0x68, 0x00E9, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x42, 0x43, 0},                                  /*  20 */
-    {0x61, 0x62, 0x63, 0x68, 0},
-    {0x61, 0x62, 0x64, 0},
-    {(UChar)0x00E4, 0x62, 0x63, 0},
-    {0x61, (UChar)0x00C6, 0x63, 0},
-    {0x61, 0x43, 0x48, 0x63, 0},  /*  primary test */
-    {0x62, 0x6c, 0x61, 0x63, 0x6b, 0x2d, 0x62, 0x69, 0x72, 0x64, 0},
-    {0x34, 0},
-    {0x35, 0},
-    {0x6f, 0x6e, 0x65, 0},
-    {0x61, 0x62, 0x63, 0},
-    {0x61, 0x42, 0x63, 0},                                  /*  30 */
-    {0x61, 0x62, 0x63, 0x68, 0},
-    {0x61, 0x62, 0x64, 0},
-    {0x61, 0x43, 0x48, 0x63, 0},                                /*  34 */
-    {0x61, 0x63, 0x65, 0x30},
-    {0x31, 0x30},
-    {0x70, (UChar)0x00EB,0x30}                                    /* 37 */
-};
-
-#ifndef INCLUDE_CALLCOLL_C
+static void TestFCDCrash(void);
 
 const UCollationResult results[] = {
     UCOL_LESS,
@@ -199,30 +126,62 @@ const UCollationResult results[] = {
     UCOL_LESS                                        /* 37 */
 };
 
-#endif
 
-const UChar testCases[][4] =
-{
-    {0x61, 0},
-    {0x41, 0},
-    {0x00e4, 0},
-    {0x00c4, 0},
-    {0x61, 0x65, 0},
-    {0x61, 0x45, 0},
-    {0x41, 0x65, 0},
-    {0x41, 0x45, 0},
-    {(UChar)0x00e6, 0},
-    {(UChar)0x00c6, 0},
-    {0x62, 0},
-    {0x63, 0},
-    {0x7a, 0}
-};
+static
+void uprv_appendByteToHexString(char *dst, uint8_t val) {
+  uint32_t len = (uint32_t)uprv_strlen(dst);
+  *(dst+len) = T_CString_itosOffset((val >> 4));
+  *(dst+len+1) = T_CString_itosOffset((val & 0xF));
+  *(dst+len+2) = 0;
+}
 
-#define COUNT_TEST_CASES 13
+/* this function makes a string with representation of a sortkey */
+static char* U_EXPORT2 sortKeyToString(const UCollator *coll, const uint8_t *sortkey, char *buffer, uint32_t *len) {
+  int32_t strength = UCOL_PRIMARY;
+  uint32_t res_size = 0;
+  UBool doneCase = FALSE;
 
-#ifndef INCLUDE_CALLCOLL_C
+  char *current = buffer;
+  const uint8_t *currentSk = sortkey;
 
-static void TestJitterbug1098(void);
+  uprv_strcpy(current, "[");
+
+  while(strength <= UCOL_QUATERNARY && strength <= coll->strength) {
+    if(strength > UCOL_PRIMARY) {
+      uprv_strcat(current, " . ");
+    }
+    while(*currentSk != 0x01 && *currentSk != 0x00) { /* print a level */
+      uprv_appendByteToHexString(current, *currentSk++);
+      uprv_strcat(current, " ");
+    }
+    if(coll->caseLevel == UCOL_ON && strength == UCOL_SECONDARY && doneCase == FALSE) {
+        doneCase = TRUE;
+    } else if(coll->caseLevel == UCOL_OFF || doneCase == TRUE || strength != UCOL_SECONDARY) {
+      strength ++;
+    }
+    uprv_appendByteToHexString(current, *currentSk++); /* This should print '01' */
+    if(strength == UCOL_QUATERNARY && coll->alternateHandling == UCOL_NON_IGNORABLE) {
+      break;
+    }
+  }
+
+  if(coll->strength == UCOL_IDENTICAL) {
+    uprv_strcat(current, " . ");
+    while(*currentSk != 0) {
+      uprv_appendByteToHexString(current, *currentSk++);
+      uprv_strcat(current, " ");
+    }
+
+    uprv_appendByteToHexString(current, *currentSk++);
+  }
+  uprv_strcat(current, "]");
+
+  if(res_size > *len) {
+    return NULL;
+  }
+
+  return buffer;
+}
 
 void addAllCollTest(TestNode** root)
 {
@@ -238,20 +197,162 @@ void addAllCollTest(TestNode** root)
     addTest(root, &TestSurrogates, "tscoll/callcoll/TestSurrogates");
     addTest(root, &TestInvalidRules, "tscoll/callcoll/TestInvalidRules");
     addTest(root, &TestJB1401, "tscoll/callcoll/TestJB1401");      
-    addTest(root, &TestJitterbug1098, "tscoll/callcoll/TestJitterbug1098");      
+    addTest(root, &TestJitterbug1098, "tscoll/callcoll/TestJitterbug1098");     
+    addTest(root, &TestFCDCrash, "tscoll/callcoll/TestFCDCrash");
 
    }
 
+UBool hasCollationElements(const char *locName) {
+
+  UErrorCode status = U_ZERO_ERROR;
+  UResourceBundle *ColEl = NULL;
+
+  UResourceBundle *loc = ures_open(NULL, locName, &status);;
+
+  if(U_SUCCESS(status)) {
+    status = U_ZERO_ERROR;
+    ColEl = ures_getByKey(loc, "collations", ColEl, &status);
+    if(status == U_ZERO_ERROR) { /* do the test - there are real elements */
+      ures_close(ColEl);
+      ures_close(loc);
+      return TRUE;
+    }
+    ures_close(ColEl);
+    ures_close(loc);
+  }
+  return FALSE;
+}
+
+static UCollationResult compareUsingPartials(UCollator *coll, const UChar source[], int32_t sLen, const UChar target[], int32_t tLen, int32_t pieceSize, UErrorCode *status) {
+  int32_t partialSKResult = 0;
+  UCharIterator sIter, tIter;
+  uint32_t sState[2], tState[2];
+  int32_t sSize = pieceSize, tSize = pieceSize;
+  /*int32_t i = 0;*/
+  uint8_t sBuf[16384], tBuf[16384];
+  if(pieceSize > 16384) {
+    log_err("Partial sortkey size buffer too small. Please consider increasing the buffer!\n");
+    *status = U_BUFFER_OVERFLOW_ERROR;
+    return UCOL_EQUAL;
+  }
+  *status = U_ZERO_ERROR;
+  sState[0] = 0; sState[1] = 0;
+  tState[0] = 0; tState[1] = 0;
+  while(sSize == pieceSize && tSize == pieceSize && partialSKResult == 0) {
+    uiter_setString(&sIter, source, sLen);
+    uiter_setString(&tIter, target, tLen);
+    sSize = ucol_nextSortKeyPart(coll, &sIter, sState, sBuf, pieceSize, status);
+    tSize = ucol_nextSortKeyPart(coll, &tIter, tState, tBuf, pieceSize, status);
+    
+    if(sState[0] != 0 || tState[0] != 0) {
+      /*log_verbose("State != 0 : %08X %08X\n", sState[0], tState[0]);*/
+    }
+    /*log_verbose("%i ", i++);*/
+    
+    partialSKResult = memcmp(sBuf, tBuf, pieceSize);
+  }
+
+  if(partialSKResult < 0) {
+      return UCOL_LESS;
+  } else if(partialSKResult > 0) {
+    return UCOL_GREATER;
+  } else {
+    return UCOL_EQUAL;
+  }
+}
+
 static void doTestVariant(UCollator* myCollation, const UChar source[], const UChar target[], UCollationResult result)
 {
     int32_t sortklen1, sortklen2, sortklenmax, sortklenmin;
     int temp=0, gSortklen1=0,gSortklen2=0;
-    UCollationResult compareResult, compareResulta, keyResult, incResult = result;
+    UCollationResult compareResult, compareResulta, keyResult, compareResultIter = result;
     uint8_t *sortKey1, *sortKey2, *sortKey1a, *sortKey2a;
     uint32_t sLen = u_strlen(source);
     uint32_t tLen = u_strlen(target);
     char buffer[256];
     uint32_t len;
+    UErrorCode status = U_ZERO_ERROR;
+    UColAttributeValue norm = ucol_getAttribute(myCollation, UCOL_NORMALIZATION_MODE, &status);
+
+    UCharIterator sIter, tIter;
+    uiter_setString(&sIter, source, sLen);
+    uiter_setString(&tIter, target, tLen);
+    compareResultIter = ucol_strcollIter(myCollation, &sIter, &tIter, &status);
+    if(compareResultIter != result) {
+      log_err("different results in iterative comparison for UTF-16 encoded strings. %s, %s\n", aescstrdup(source,-1), aescstrdup(target,-1));
+    }
+
+    /* convert the strings to UTF-8 and do try comparing with char iterator */
+    if(QUICK <= 0) { /*!QUICK*/
+      char utf8Source[256], utf8Target[256];
+      int32_t utf8SourceLen = 0, utf8TargetLen = 0;
+      u_strToUTF8(utf8Source, 256, &utf8SourceLen, source, sLen, &status);
+      if(U_FAILURE(status)) { /* probably buffer is not big enough */
+        log_verbose("Src UTF-8 buffer too small! Will not compare!\n");
+      } else {
+        u_strToUTF8(utf8Target, 256, &utf8TargetLen, target, tLen, &status);
+        if(U_SUCCESS(status)) { /* probably buffer is not big enough */
+          UCollationResult compareResultUTF8 = result, compareResultUTF8Norm = result;
+          /*UCharIterator sIter, tIter;*/
+          /*log_verbose("Strings converted to UTF-8:%s, %s\n", aescstrdup(source,-1), aescstrdup(target,-1));*/
+          uiter_setUTF8(&sIter, utf8Source, utf8SourceLen);
+          uiter_setUTF8(&tIter, utf8Target, utf8TargetLen);
+       /*uiter_setString(&sIter, source, sLen);
+      uiter_setString(&tIter, target, tLen);*/
+          compareResultUTF8 = ucol_strcollIter(myCollation, &sIter, &tIter, &status);
+          ucol_setAttribute(myCollation, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
+          sIter.move(&sIter, 0, UITER_START);
+          tIter.move(&tIter, 0, UITER_START);
+          compareResultUTF8Norm = ucol_strcollIter(myCollation, &sIter, &tIter, &status);
+          ucol_setAttribute(myCollation, UCOL_NORMALIZATION_MODE, norm, &status);
+          if(compareResultUTF8 != compareResultIter) {
+            log_err("different results in iterative comparison for UTF-16 and UTF-8 encoded strings. %s, %s\n", aescstrdup(source,-1), aescstrdup(target,-1));
+          }
+          if(compareResultUTF8 != compareResultUTF8Norm) {
+            log_err("different results in iterative when normalization is turned on with UTF-8 strings. %s, %s\n", aescstrdup(source,-1), aescstrdup(target,-1));
+          }
+        } else {
+          log_verbose("Target UTF-8 buffer too small! Did not compare!\n");
+        }
+        if(U_FAILURE(status)) {
+          log_verbose("UTF-8 strcoll failed! Ignoring result\n");
+        }
+      }
+    }
+
+    /* testing the partial sortkeys */
+    if(1) { /*!QUICK*/
+      int32_t i = 0;
+      int32_t partialSizes[] = { 3, 1, 2, 4, 8, 20, 80 }; /* just size 3 in the quick mode */
+      int32_t partialSizesSize = 1;
+      if(QUICK <= 0) {
+        partialSizesSize = 7;
+      }
+      /*log_verbose("partial sortkey test piecesize=");*/
+      for(i = 0; i < partialSizesSize; i++) {
+        UCollationResult partialSKResult = result, partialNormalizedSKResult = result;
+        /*log_verbose("%i ", partialSizes[i]);*/
+
+        partialSKResult = compareUsingPartials(myCollation, source, sLen, target, tLen, partialSizes[i], &status);
+        if(partialSKResult != result) {
+          log_err("Partial sortkey comparison returned wrong result (%i exp. %i): %s, %s (size %i)\n", 
+            partialSKResult, result,
+            aescstrdup(source,-1), aescstrdup(target,-1), partialSizes[i]);
+        }
+
+        if(QUICK <= 0 && norm != UCOL_ON) {
+          /*log_verbose("N ");*/
+          ucol_setAttribute(myCollation, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
+          partialNormalizedSKResult = compareUsingPartials(myCollation, source, sLen, target, tLen, partialSizes[i], &status);
+          ucol_setAttribute(myCollation, UCOL_NORMALIZATION_MODE, norm, &status);
+          if(partialSKResult != partialNormalizedSKResult) {
+            log_err("Partial sortkey comparison gets different result when normalization is on: %s, %s (size %i)\n", 
+              aescstrdup(source,-1), aescstrdup(target,-1), partialSizes[i]);
+          }
+        }
+      }
+      /*log_verbose("\n");*/
+    }
 
     
     compareResult  = ucol_strcoll(myCollation, source, sLen, target, tLen);
@@ -289,11 +390,11 @@ static void doTestVariant(UCollator* myCollation, const UChar source[], const UC
     gSortklen2 = uprv_strlen((const char *)sortKey2)+1;
     if(sortklen1 != gSortklen1){
         log_err("SortKey length does not match Expected: %i Got: %i\n",sortklen1, gSortklen1);
-        log_verbose("Generated sortkey: %s\n", ucol_sortKeyToString(myCollation, sortKey1, buffer, &len));
+        log_verbose("Generated sortkey: %s\n", sortKeyToString(myCollation, sortKey1, buffer, &len));
     }
     if(sortklen2!= gSortklen2){
         log_err("SortKey length does not match Expected: %i Got: %i\n", sortklen2, gSortklen2);
-        log_verbose("Generated sortkey: %s\n", ucol_sortKeyToString(myCollation, sortKey2, buffer, &len));
+        log_verbose("Generated sortkey: %s\n", sortKeyToString(myCollation, sortKey2, buffer, &len));
     }
 
     if(temp < 0) {
@@ -305,7 +406,7 @@ static void doTestVariant(UCollator* myCollation, const UChar source[], const UC
     else {
         keyResult = UCOL_EQUAL;
     }
-    reportCResult( source, target, sortKey1, sortKey2, compareResult, keyResult, incResult, result );
+    reportCResult( source, target, sortKey1, sortKey2, compareResult, keyResult, compareResultIter, result );
     free(sortKey1);
     free(sortKey2);
     free(sortKey1a);
@@ -315,16 +416,270 @@ static void doTestVariant(UCollator* myCollation, const UChar source[], const UC
 
 void doTest(UCollator* myCollation, const UChar source[], const UChar target[], UCollationResult result)
 {
-  doTestVariant(myCollation, source, target, result);
-  if(result == UCOL_LESS) {
-    doTestVariant(myCollation, target, source, UCOL_GREATER);
-  } else if(result == UCOL_GREATER) {
-    doTestVariant(myCollation, target, source, UCOL_LESS);
+  if(myCollation) {
+    doTestVariant(myCollation, source, target, result);
+    if(result == UCOL_LESS) {
+      doTestVariant(myCollation, target, source, UCOL_GREATER);
+    } else if(result == UCOL_GREATER) {
+      doTestVariant(myCollation, target, source, UCOL_LESS);
+    } else {
+      doTestVariant(myCollation, target, source, UCOL_EQUAL);
+    }
   } else {
-    doTestVariant(myCollation, target, source, UCOL_EQUAL);
+    log_data_err("No collator! Any data around?\n");
+  }
+}
+
+
+/**
+ * Return an integer array containing all of the collation orders
+ * returned by calls to next on the specified iterator
+ */
+int32_t* getOrders(UCollationElements *iter, int32_t *orderLength)
+{
+    UErrorCode status;
+    int32_t order;
+    int32_t maxSize = 100;
+    int32_t size = 0;
+    int32_t *temp;
+    int32_t *orders =(int32_t*)malloc(sizeof(int32_t) * maxSize);
+    status= U_ZERO_ERROR;
+
+
+    while ((order=ucol_next(iter, &status)) != UCOL_NULLORDER)
+    {
+        if (size == maxSize)
+        {
+            maxSize *= 2;
+            temp = (int32_t*)malloc(sizeof(int32_t) * maxSize);
+
+            memcpy(temp, orders, size * sizeof(int32_t));
+            free(orders);
+            orders = temp;
+
+        }
+
+        orders[size++] = order;
+    }
+
+    if (maxSize > size && size > 0)
+    {
+        temp = (int32_t*)malloc(sizeof(int32_t) * size);
+
+        memcpy(temp, orders, size * sizeof(int32_t));
+        free(orders);
+        orders = temp;
+
+
+    }
+
+    *orderLength = size;
+    return orders;
+}
+
+
+void 
+backAndForth(UCollationElements *iter)
+{
+    /* Run through the iterator forwards and stick it into an array */
+    int32_t index, o;
+    UErrorCode status = U_ZERO_ERROR;
+    int32_t orderLength = 0;
+    int32_t *orders;
+    orders= getOrders(iter, &orderLength);
+
+
+    /* Now go through it backwards and make sure we get the same values */
+    index = orderLength;
+    ucol_reset(iter);
+
+    /* synwee : changed */
+    while ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
+    {
+        if (o != orders[-- index])
+        {
+        if (o == 0)
+          index ++;
+        else
+        {
+          while (index > 0 && orders[-- index] == 0)
+          {
+          }
+          if (o != orders[index])
+          {
+            log_err("Mismatch at index : 0x%x\n", index);
+            return;
+        }
+
+        }
+      }
+    }
+
+    while (index != 0 && orders[index - 1] == 0) {
+      index --;
+    }
+
+    if (index != 0)
+    {
+        log_err("Didn't get back to beginning - index is %d\n", index);
+
+        ucol_reset(iter);
+        log_err("\nnext: ");
+        if ((o = ucol_next(iter, &status)) != UCOL_NULLORDER)
+        {
+            log_err("Error at %x\n", o);
+        }
+        log_err("\nprev: ");
+        if ((o = ucol_previous(iter, &status)) != UCOL_NULLORDER)
+        {
+            log_err("Error at %x\n", o);
+        }
+        log_verbose("\n");
+    }
+
+    free(orders);
+}
+
+void genericOrderingTestWithResult(UCollator *coll, const char *s[], uint32_t size, UCollationResult result) {
+  UChar t1[2048] = {0};
+  UChar t2[2048] = {0};
+  UCollationElements *iter;
+  UErrorCode status = U_ZERO_ERROR;
+
+  uint32_t i = 0, j = 0;
+  log_verbose("testing sequence:\n");
+  for(i = 0; i < size; i++) {
+    log_verbose("%s\n", s[i]);
   }
 
+  iter = ucol_openElements(coll, t1, u_strlen(t1), &status);
+  if (U_FAILURE(status)) {
+    log_err("Creation of iterator failed\n");
+  }
+  for(i = 0; i < size-1; i++) {
+    for(j = i+1; j < size; j++) {
+      u_unescape(s[i], t1, 2048);
+      u_unescape(s[j], t2, 2048);
+      doTest(coll, t1, t2, result);
+      /* synwee : added collation element iterator test */
+      ucol_setText(iter, t1, u_strlen(t1), &status);
+      backAndForth(iter);
+      ucol_setText(iter, t2, u_strlen(t2), &status);
+      backAndForth(iter);
+    }
+  }
+  ucol_closeElements(iter);
+}
 
+void genericOrderingTest(UCollator *coll, const char *s[], uint32_t size) {
+  genericOrderingTestWithResult(coll, s, size, UCOL_LESS);
+}
+
+void genericLocaleStarter(const char *locale, const char *s[], uint32_t size) {
+  UErrorCode status = U_ZERO_ERROR;
+  UCollator *coll = ucol_open(locale, &status);
+
+  log_verbose("Locale starter for %s\n", locale);
+
+  if(U_SUCCESS(status)) {
+    genericOrderingTest(coll, s, size);
+  } else if(status == U_FILE_ACCESS_ERROR) {
+    log_data_err("Is your data around?\n");
+    return;
+  } else {
+    log_err("Unable to open collator for locale %s\n", locale);
+  }
+  ucol_close(coll);
+}
+
+void genericLocaleStarterWithResult(const char *locale, const char *s[], uint32_t size, UCollationResult result) {
+  UErrorCode status = U_ZERO_ERROR;
+  UCollator *coll = ucol_open(locale, &status);
+
+  log_verbose("Locale starter for %s\n", locale);
+
+  if(U_SUCCESS(status)) {
+    genericOrderingTestWithResult(coll, s, size, result);
+  } else if(status == U_FILE_ACCESS_ERROR) {
+    log_data_err("Is your data around?\n");
+    return;
+  } else {
+    log_err("Unable to open collator for locale %s\n", locale);
+  }
+  ucol_close(coll);
+}
+
+/* currently not used with options */
+void genericRulesStarterWithOptionsAndResult(const char *rules, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+  UErrorCode status = U_ZERO_ERROR;
+  UChar rlz[RULE_BUFFER_LEN] = { 0 };
+  uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
+  uint32_t i;
+
+  UCollator *coll = ucol_openRules(rlz, rlen, UCOL_DEFAULT, UCOL_DEFAULT,NULL, &status);
+
+  log_verbose("Rules starter for %s\n", rules);
+
+  if(U_SUCCESS(status)) {
+    log_verbose("Setting attributes\n");
+    for(i = 0; i < attsize; i++) {
+      ucol_setAttribute(coll, attrs[i], values[i], &status);
+    }
+
+    genericOrderingTestWithResult(coll, s, size, result);
+  } else {
+    log_err("Unable to open collator with rules %s\n", rules);
+  }
+  ucol_close(coll);
+}
+
+void genericLocaleStarterWithOptionsAndResult(const char *locale, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize, UCollationResult result) {
+  UErrorCode status = U_ZERO_ERROR;
+  uint32_t i;
+
+  UCollator *coll = ucol_open(locale, &status);
+
+  log_verbose("Locale starter for %s\n", locale);
+
+  if(U_SUCCESS(status)) {
+
+    log_verbose("Setting attributes\n");
+    for(i = 0; i < attsize; i++) {
+      ucol_setAttribute(coll, attrs[i], values[i], &status);
+    }
+
+    genericOrderingTestWithResult(coll, s, size, result);
+  } else {
+    log_err("Unable to open collator for locale %s\n", locale);
+  }
+  ucol_close(coll);
+}
+
+void genericLocaleStarterWithOptions(const char *locale, const char *s[], uint32_t size, const UColAttribute *attrs, const UColAttributeValue *values, uint32_t attsize) {
+  genericLocaleStarterWithOptionsAndResult(locale, s, size, attrs, values, attsize, UCOL_LESS);
+}
+
+void genericRulesStarterWithResult(const char *rules, const char *s[], uint32_t size, UCollationResult result) {
+  UErrorCode status = U_ZERO_ERROR;
+  UChar rlz[RULE_BUFFER_LEN] = { 0 };
+  uint32_t rlen = u_unescape(rules, rlz, RULE_BUFFER_LEN);
+
+  UCollator *coll = NULL;
+  coll = ucol_openRules(rlz, rlen, UCOL_DEFAULT, UCOL_DEFAULT,NULL, &status);
+  log_verbose("Rules starter for %s\n", rules);
+
+  if(U_SUCCESS(status)) {
+    genericOrderingTestWithResult(coll, s, size, result);
+    ucol_close(coll);
+  } else if(status == U_FILE_ACCESS_ERROR) {
+    log_data_err("Is your data around?\n");
+  } else {
+    log_err("Unable to open collator with rules %s\n", rules);
+  }
+}
+
+void genericRulesStarter(const char *rules, const char *s[], uint32_t size) {
+  genericRulesStarterWithResult(rules, s, size, UCOL_LESS);
 }
 
 static void TestTertiary()
@@ -684,12 +1039,14 @@ static void TestVariableTop(void)
 }
 
 /**
-* Tests surrogate support.
-*/
+  * Tests surrogate support.
+  * NOTE: This test used \\uD801\\uDC01 pair, which is now assigned to Desseret
+  * Therefore, another (unassigned) code point was used for this test.
+  */
 static void TestSurrogates(void)
 {
     const char       *str          = 
-                              "&z<'\\uD800\\uDC00'<'\\uD801\\uDC01\\u0308'<A";
+                              "&z<'\\uD800\\uDC00'<'\\uD800\\uDC0A\\u0308'<A";
           int         len          = strlen(str);
           int         rlen         = 0;
           UChar      *rules;
@@ -697,9 +1054,9 @@ static void TestSurrogates(void)
           UCollator  *enCollation;
           UErrorCode  status       = U_ZERO_ERROR;
           UChar       source[][4]    = 
-          {{'z', 0, 0}, {0xD800, 0xDC00, 0}, {0xD801, 0xDC01, 0x0308, 0}};
+          {{'z', 0, 0}, {0xD800, 0xDC00, 0}, {0xD800, 0xDC0A, 0x0308, 0}, {0xD800, 0xDC02}};
           UChar       target[][4]    = 
-          {{0xD800, 0xDC00, 0}, {0xD801, 0xDC01, 0x0308, 0}, {'A', 0, 0}};
+          {{0xD800, 0xDC00, 0}, {0xD800, 0xDC0A, 0x0308, 0}, {'A', 0, 0}, {0xD800, 0xDC03}};
           int         count        = 0;
           uint8_t enresult[20], myresult[20];
           int enlen, mylen;
@@ -709,7 +1066,7 @@ static void TestSurrogates(void)
     rlen = u_unescape(str, rules, len);
     
     enCollation = ucol_open("en_US", &status);
-    myCollation = ucol_openRules(rules, rlen - 1, UCOL_OFF, 
+    myCollation = ucol_openRules(rules, rlen, UCOL_OFF, 
                                  UCOL_TERTIARY,NULL, &status);
     if (U_FAILURE(status)) {
         log_err("ERROR: in creation of rule based collator :%s\n", 
@@ -731,14 +1088,14 @@ static void TestSurrogates(void)
     log_verbose("start of tailored collation supplementary characters test\n");
     count = 0;
     /* tests getting collation elements for surrogates for tailored rules */
-    while (count < 3) {
+    while (count < 4) {
         doTest(myCollation, source[count], target[count], UCOL_LESS);
         count ++;
     }
 
-    /* tests that \uD801\uDC01 still has the same value, not changed */
-    enlen = ucol_getSortKey(enCollation, source[2], 2, enresult, 20);
-    mylen = ucol_getSortKey(myCollation, source[2], 2, myresult, 20);
+    /* tests that \uD800\uDC02 still has the same value, not changed */
+    enlen = ucol_getSortKey(enCollation, source[3], 2, enresult, 20);
+    mylen = ucol_getSortKey(myCollation, source[3], 2, myresult, 20);
     if (enlen != mylen ||
         uprv_memcmp(enresult, myresult, enlen) != 0) {
         log_verbose("Failed : non-tailored supplementary characters should have the same value\n");
@@ -781,9 +1138,9 @@ TestInvalidRules(){
         UParseError parseError;
         UErrorCode status = U_ZERO_ERROR;
         UCollator* coll=0;
-        u_charsToUChars(rulesArr[i],rules,1000);
-        u_charsToUChars(preContextArr[i],preContextExp,1000);
-        u_charsToUChars(postContextArr[i],postContextExp,1000);
+        u_charsToUChars(rulesArr[i],rules,uprv_strlen(rulesArr[i])+1);
+        u_charsToUChars(preContextArr[i],preContextExp,uprv_strlen(preContextArr[i])+1);
+        u_charsToUChars(postContextArr[i],postContextExp,uprv_strlen(postContextArr[i])+1);
         /* clean up stuff in parseError */
         u_memset(parseError.preContext,0x0000,U_PARSE_CONTEXT_LEN);      
         u_memset(parseError.postContext,0x0000,U_PARSE_CONTEXT_LEN);
@@ -835,10 +1192,14 @@ TestJitterbug1098(){
         u_uastrcpy(rule, rules[i]);
         c1 = ucol_openRules(rule, u_strlen(rule), UCOL_OFF, UCOL_DEFAULT_STRENGTH, &parseError, &status);
         if(U_FAILURE(status)){
-            u_UCharsToChars(parseError.preContext,preContext,20);
-            u_UCharsToChars(parseError.postContext,postContext,20);
             log_err("Could not parse the rules syntax. Error: %s ", u_errorName(status));
-            log_verbose("\n\tPre-Context: %s \n\tPost-Context:%s \n",preContext,postContext);
+
+            if (status == U_PARSE_ERROR) {
+                u_UCharsToChars(parseError.preContext,preContext,20);
+                u_UCharsToChars(parseError.postContext,postContext,20);
+                log_verbose("\n\tPre-Context: %s \n\tPost-Context:%s \n",preContext,postContext);
+            }
+
             return;
         }
         X[0] = input[i][0];
@@ -848,4 +1209,49 @@ TestJitterbug1098(){
     }
 }
 
-#endif
+/*  These tests do cleanup and reinitialize ICU in the course of their operation.
+ *    The ICU data directory must be preserved across these operations.
+ *    Here is a helper function to assist with that.
+ */
+static char *safeGetICUDataDirectory() {
+    const char *dataDir = u_getDataDirectory();  /* Returned string vanashes with u_cleanup */
+    char *retStr = NULL;
+    if (dataDir != NULL) {
+        retStr = (char *)malloc(strlen(dataDir)+1);
+        strcpy(retStr, dataDir);
+    }
+    return retStr;
+}
+
+
+static void
+TestFCDCrash(void) {
+    static const char *test[] = {
+    "Gr\\u00F6\\u00DFe",
+    "Grossist"
+    };
+
+    char *icuDataDir = safeGetICUDataDirectory();
+    UErrorCode status = U_ZERO_ERROR;
+    UCollator *coll = ucol_open("es", &status);
+    if(U_FAILURE(status)) {
+        log_err("Couldn't open collator\n");
+        return;
+    }
+    ucol_close(coll);
+    coll = NULL;
+    u_cleanup();
+    u_setDataDirectory(icuDataDir);
+    coll = ucol_open("de_DE", &status);
+    if(U_FAILURE(status)) {
+        log_err("Couldn't open collator\n");
+        return;
+    }
+    ucol_setAttribute(coll, UCOL_NORMALIZATION_MODE, UCOL_ON, &status);
+    genericOrderingTest(coll, test, 2);
+    ucol_close(coll);
+    free(icuDataDir);
+
+}
+
+#endif /* #if !UCONFIG_NO_COLLATION */
