@@ -1,13 +1,9 @@
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2002, International Business Machines Corporation and
+ * Copyright (c) 1997-2005, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
-#include <stdlib.h>
-#include <time.h>
-#include <string.h>
-
 #include "unicode/utypes.h"
 
 #include "cstring.h"
@@ -15,6 +11,10 @@
 #include "unicode/uniset.h"
 #include "unicode/resbund.h"
 #include "restest.h"
+
+#include <stdlib.h>
+#include <time.h>
+#include <string.h>
 #include <limits.h>
 
 //***************************************************************************************
@@ -85,7 +85,7 @@ itoa(int32_t i, char* buf)
 
 // Array of our test objects
 
-struct
+static struct
 {
     const char* name;
     Locale *locale;
@@ -103,12 +103,12 @@ param[] =
     { "root",       NULL,   U_ZERO_ERROR,             e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } },
     { "te",         NULL,   U_ZERO_ERROR,             e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE } },
     { "te_IN",      NULL,   U_ZERO_ERROR,             e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE } },
-    { "te_NE",      NULL,   U_USING_FALLBACK_ERROR,   e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE } },
-    { "te_IN_NE",   NULL,   U_USING_FALLBACK_ERROR,   e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE } },
-    { "ne",         NULL,   U_USING_DEFAULT_ERROR,    e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } }
+    { "te_NE",      NULL,   U_USING_FALLBACK_WARNING, e_te,        { FALSE, TRUE, FALSE }, { TRUE, TRUE, FALSE } },
+    { "te_IN_NE",   NULL,   U_USING_FALLBACK_WARNING, e_te_IN,     { FALSE, FALSE, TRUE }, { TRUE, TRUE, TRUE } },
+    { "ne",         NULL,   U_USING_DEFAULT_WARNING,  e_Root,      { TRUE, FALSE, FALSE }, { TRUE, FALSE, FALSE } }
 };
 
-int32_t bundles_count = sizeof(param) / sizeof(param[0]);
+static const int32_t bundles_count = sizeof(param) / sizeof(param[0]);
 
 //***************************************************************************************
 
@@ -156,8 +156,7 @@ int32_t randi(int32_t n)
 */
 ResourceBundleTest::ResourceBundleTest()
 : pass(0),
-  fail(0),
-  OUT(it_out)
+  fail(0)
 {
     if (param[5].locale == NULL) {
         param[0].locale = new Locale("root");
@@ -187,6 +186,8 @@ void ResourceBundleTest::runIndexedTest( int32_t index, UBool exec, const char*
     case 0: name = "TestResourceBundles"; if (exec) TestResourceBundles(); break;
     case 1: name = "TestConstruction"; if (exec) TestConstruction(); break;
     case 2: name = "TestExemplar"; if (exec) TestExemplar(); break;
+    case 3: name = "TestGetSize"; if (exec) TestGetSize(); break;
+    case 4: name = "TestGetLocaleByType"; if (exec) TestGetLocaleByType(); break;
         default: name = ""; break; //needed to end loop
     }
 }
@@ -218,7 +219,7 @@ ResourceBundleTest::TestConstruction()
         testdatapath=loadTestData(err);
         if(U_FAILURE(err))
         {
-            errln("Could not load testdata.dat %s " + UnicodeString(u_errorName(err)));
+            errln("Could not load testdata.dat " + UnicodeString(testdatapath) +  ", " + UnicodeString(u_errorName(err)));
             return;
         }
         ResourceBundle  test1((UnicodeString)testdatapath, err);
@@ -262,39 +263,6 @@ ResourceBundleTest::TestConstruction()
         delete[] versionID1;
         delete[] versionID2;
     }
-    {
-        UErrorCode   err = U_ZERO_ERROR;
-        const char* testdatapath;
-        Locale       locale("te", "IN");
-
-        testdatapath=loadTestData(err);
-        if(U_FAILURE(err))
-        {
-            errln("Could not load testdata.dat %s " + UnicodeString(u_errorName(err)));
-            return;
-        }
-
-
-        wchar_t* wideDirectory = new wchar_t[256];
-        mbstowcs(wideDirectory, testdatapath, 256);
-        //mbstowcs(wideDirectory, "c:\\icu\\icu\\source\\test\\testdata\\testdata", 256);
-
-        ResourceBundle  test2(wideDirectory, locale, err);
-
-        UnicodeString   result2(test2.getStringEx("string_in_Root_te_te_IN", err));
-
-        if (U_FAILURE(err)) {
-            errln("Something threw an error in TestConstruction()");
-            return;
-        }
-
-        logln("for string_in_Root_te_te_IN, te_IN.txt had " + result2);
-
-        if (result2 != "TE_IN")
-            errln("Construction test failed; run verbose for more information");
-
-        delete[] wideDirectory;
-    }
 }
 
 //***************************************************************************************
@@ -358,9 +326,9 @@ ResourceBundleTest::testTag(const char* frag,
                 if(j == actual_bundle) /* it's in the same bundle OR it's a nonexistent=default bundle (5) */
                   expected_resource_status = U_ZERO_ERROR;
                 else if(j == 0)
-                  expected_resource_status = U_USING_DEFAULT_ERROR;
+                  expected_resource_status = U_USING_DEFAULT_WARNING;
                 else
-                  expected_resource_status = U_USING_FALLBACK_ERROR;
+                  expected_resource_status = U_USING_FALLBACK_WARNING;
                 
                 break;
             }
@@ -512,6 +480,124 @@ ResourceBundleTest::TestExemplar(){
     logln("Number of installed locales with exemplar characters that could be tested: %d",num);
 
 }
-            
+
+void 
+ResourceBundleTest::TestGetSize(void) 
+{
+    const struct {
+        const char* key;
+        int32_t size;
+    } test[] = {
+        { "zerotest", 1},
+        { "one", 1},
+        { "importtest", 1},
+        { "integerarray", 1},
+        { "emptyarray", 0},
+        { "emptytable", 0},
+        { "emptystring", 1}, /* empty string is still a string */
+        { "emptyint", 1}, 
+        { "emptybin", 1},
+        { "testinclude", 1},
+        { "collations", 1}, /* not 2 - there is hidden %%CollationBin */
+    };
+    
+    UErrorCode status = U_ZERO_ERROR;
+    
+    const char* testdatapath = loadTestData(status);
+    int32_t i = 0, j = 0;
+    int32_t size = 0;
+    
+    if(U_FAILURE(status))
+    {
+        err("Could not load testdata.dat %s\n", u_errorName(status));
+        return;
+    }
+    
+    ResourceBundle rb(testdatapath, "testtypes", status);
+    if(U_FAILURE(status))
+    {
+        err("Could not testtypes resource bundle %s\n", u_errorName(status));
+        return;
+    }
+    
+    for(i = 0; i < (int32_t)(sizeof(test)/sizeof(test[0])); i++) {
+        ResourceBundle res = rb.get(test[i].key, status);
+        if(U_FAILURE(status))
+        {
+            err("Couldn't find the key %s. Error: %s\n", u_errorName(status));
+            return;
+        }
+        size = res.getSize();
+        if(size != test[i].size) {
+            err("Expected size %i, got size %i for key %s\n", test[i].size, size, test[i].key);
+            for(j = 0; j < size; j++) {
+                ResourceBundle helper = res.get(j, status);
+                err("%s\n", helper.getKey());
+            }
+        }
+    }
+}
+
+void 
+ResourceBundleTest::TestGetLocaleByType(void) 
+{
+    const struct {
+        const char *requestedLocale;
+        const char *resourceKey;
+        const char *validLocale;
+        const char *actualLocale;
+    } test[] = {
+        { "te_IN_BLAH", "string_only_in_te_IN", "te_IN", "te_IN" },
+        { "te_IN_BLAH", "string_only_in_te", "te_IN", "te" },
+        { "te_IN_BLAH", "string_only_in_Root", "te_IN", "root" },
+        { "te_IN_BLAH_01234567890_01234567890_01234567890_01234567890_01234567890_01234567890", "array_2d_only_in_Root", "te_IN", "root" },
+        { "te_IN_BLAH@currency=euro", "array_2d_only_in_te_IN", "te_IN", "te_IN" },
+        { "te_IN_BLAH@calendar=thai;collation=phonebook", "array_2d_only_in_te", "te_IN", "te" }
+    };
+    
+    UErrorCode status = U_ZERO_ERROR;
+    
+    const char* testdatapath = loadTestData(status);
+    int32_t i = 0;
+    Locale locale;
+    
+    if(U_FAILURE(status))
+    {
+        err("Could not load testdata.dat %s\n", u_errorName(status));
+        return;
+    }
+    
+    for(i = 0; i < (int32_t)(sizeof(test)/sizeof(test[0])); i++) {
+        ResourceBundle rb(testdatapath, test[i].requestedLocale, status);
+        if(U_FAILURE(status))
+        {
+            err("Could not open resource bundle %s (error %s)\n", test[i].requestedLocale, u_errorName(status));
+            status = U_ZERO_ERROR;
+            continue;
+        }
+        
+        ResourceBundle res = rb.get(test[i].resourceKey, status);
+        if(U_FAILURE(status))
+        {
+            err("Couldn't find the key %s. Error: %s\n", test[i].resourceKey, u_errorName(status));
+            status = U_ZERO_ERROR;
+            continue;
+        }
+        
+        locale = res.getLocale(ULOC_REQUESTED_LOCALE, status);
+        if(locale != Locale::getDefault()) {
+            err("Expected requested locale to be %s. Got %s\n", test[i].requestedLocale, locale.getName());
+        }
+        locale = res.getLocale(ULOC_VALID_LOCALE, status);
+        if(strcmp(locale.getName(), test[i].validLocale) != 0) {
+            err("Expected valid locale to be %s. Got %s\n", test[i].requestedLocale, locale.getName());
+        }
+        locale = res.getLocale(ULOC_ACTUAL_LOCALE, status);
+        if(strcmp(locale.getName(), test[i].actualLocale) != 0) {
+            err("Expected actual locale to be %s. Got %s\n", test[i].requestedLocale, locale.getName());
+        }
+    }
+}
+
 //eof
 
