@@ -1,13 +1,17 @@
-/********************************************************************
+/***********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
- * others. All Rights Reserved.
- ********************************************************************/
+ * Copyright (c) 1997-2005, International Business Machines Corporation
+ * and others. All Rights Reserved.
+ ***********************************************************************/
 
 
 #ifndef _TESTMESSAGEFORMAT
 #define _TESTMESSAGEFORMAT
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
+
 #include "unicode/unistr.h"
 #include "unicode/fmtable.h"
 #include "intltest.h"
@@ -53,6 +57,17 @@ public:
      **/
     void testMsgFormatChoice(/* char* par */);
 
+    /**
+     * Verify that MessageFormat accomodates more than 10 arguments
+     * and more than 10 subformats.
+     */
+    void TestUnlimitedArgsAndSubformats();
+
+    /**
+     * Test RBNF extensions to MessageFormat.
+     */
+    void TestRBNF();
+
     //
     /** 
      * ------------ API tests ----------
@@ -61,6 +76,7 @@ public:
      * they often test other methods as well.
      **/
     void testCopyConstructor(void);
+    void testCopyConstructor2(void);
     void testAssignment(void);
     void testClone(void);
     void testEquals(void);
@@ -69,9 +85,12 @@ public:
     void testFormat(void);
     void testParse(void);
     void testAdopt(void);
+    void TestTurkishCasing(void);
+    void testAutoQuoteApostrophe(void);
 
 private:
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
 
 #endif
