@@ -1,15 +1,22 @@
-/********************************************************************
+/***********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
- * others. All Rights Reserved.
- ********************************************************************/
+ * Copyright (c) 1997-2006, International Business Machines Corporation
+ * and others. All Rights Reserved.
+ ***********************************************************************/
+
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
 
 #include "unicode/timezone.h"
 #include "unicode/simpletz.h"
 #include "unicode/calendar.h"
 #include "unicode/gregocal.h"
 #include "unicode/resbund.h"
+#include "unicode/strenum.h"
 #include "tztest.h"
+#include "cmemory.h"
+#include "putilimp.h"
 
 #define CASE(id,test) case id:                               \
                           name = #test;                      \
@@ -37,8 +44,10 @@ void TimeZoneTest::runIndexedTest( int32_t index, UBool exec, const char* &name,
         CASE(7, TestDisplayName);
         CASE(8, TestDSTSavings);
         CASE(9, TestAlternateRules);
-        CASE(10,TestCountries);
-        default: name = ""; break;
+        CASE(10,TestCountries); 
+        CASE(11,TestHistorical);
+        CASE(12,TestEquivalentIDs);
+       default: name = ""; break;
     }
 }
 
@@ -75,14 +84,15 @@ TimeZoneTest::TestGenericAPI()
     if (!(copy == *zoneclone)) errln("FAIL: assignment operator or operator== failed");
 
     TimeZone* saveDefault = TimeZone::createDefault();
-    TimeZone* pstZone = TimeZone::createTimeZone("PST");
+    logln((UnicodeString)"TimeZone::createDefault() => " + saveDefault->getID(id));
+    //TimeZone* pstZone = TimeZone::createTimeZone("PST");
 
-    logln("call u_t_timezone() which uses the host");
+    logln("call uprv_timezone() which uses the host");
     logln("to get the difference in seconds between coordinated universal");
     logln("time and local time. E.g., -28,800 for PST (GMT-8hrs)");
 
     int32_t tzoffset = uprv_timezone();
-    logln(UnicodeString("Value returned from t_timezone = ") + tzoffset);
+    logln(UnicodeString("Value returned from uprv_timezone = ") + tzoffset);
     // Invert sign because UNIX semantics are backwards
     if (tzoffset < 0)
         tzoffset = -tzoffset;
@@ -93,7 +103,7 @@ TimeZoneTest::TestGenericAPI()
     //}
 
     if (tzoffset != 28800) {
-        logln("***** WARNING: If testing in the PST timezone, t_timezone should return 28800! *****");
+        logln("***** WARNING: If testing in the PST timezone, uprv_timezone should return 28800! *****");
     }
     if ((tzoffset % 1800 != 0)) {
       errln("FAIL: t_timezone may be incorrect. It is not a multiple of 30min. It is %d", tzoffset);
@@ -107,7 +117,7 @@ TimeZoneTest::TestGenericAPI()
     TimeZone::adoptDefault(saveDefault);
     delete defaultzone;
     delete zoneclone;
-    delete pstZone;
+    //delete pstZone;
 }
 
 // ---------------------------------------------------------------------------------
@@ -129,10 +139,10 @@ TimeZoneTest::TestRuleAPI()
     GregorianCalendar *gc = new GregorianCalendar(*zone, status);
     if (failure(status, "new GregorianCalendar")) return;
     gc->clear();
-    gc->set(1990, Calendar::MARCH, 1);
+    gc->set(1990, UCAL_MARCH, 1);
     UDate marchOneStd = gc->getTime(status); // Local Std time midnight
     gc->clear();
-    gc->set(1990, Calendar::JULY, 1);
+    gc->set(1990, UCAL_JULY, 1);
     UDate julyOneStd = gc->getTime(status); // Local Std time midnight
     if (failure(status, "GregorianCalendar::getTime")) return;
 
@@ -140,8 +150,8 @@ TimeZoneTest::TestRuleAPI()
     int32_t startHour = (int32_t)(2.25 * 3600000);
     int32_t endHour   = (int32_t)(3.5  * 3600000);
 
-    zone->setStartRule(Calendar::MARCH, 1, 0, startHour, status);
-    zone->setEndRule  (Calendar::JULY,  1, 0, endHour, status);
+    zone->setStartRule(UCAL_MARCH, 1, 0, startHour, status);
+    zone->setEndRule  (UCAL_JULY,  1, 0, endHour, status);
 
     delete gc;
     gc = new GregorianCalendar(*zone, status);
@@ -168,8 +178,8 @@ TimeZoneTest::TestRuleAPI()
           " = " + dateToString(expJulyOne));
     }
 
-    testUsingBinarySearch(zone, date(90, Calendar::JANUARY, 1), date(90, Calendar::JUNE, 15), marchOne);
-    testUsingBinarySearch(zone, date(90, Calendar::JUNE, 1), date(90, Calendar::DECEMBER, 31), julyOne);
+    testUsingBinarySearch(*zone, date(90, UCAL_JANUARY, 1), date(90, UCAL_JUNE, 15), marchOne);
+    testUsingBinarySearch(*zone, date(90, UCAL_JUNE, 1), date(90, UCAL_DECEMBER, 31), julyOne);
 
     if (zone->inDaylightTime(marchOne - 1000, status) ||
         !zone->inDaylightTime(marchOne, status))
@@ -189,25 +199,56 @@ TimeZoneTest::TestRuleAPI()
 }
 
 void
-TimeZoneTest::testUsingBinarySearch(SimpleTimeZone* tz, UDate min, UDate max, UDate expectedBoundary)
+TimeZoneTest::findTransition(const TimeZone& tz,
+                             UDate min, UDate max) {
+    UErrorCode ec = U_ZERO_ERROR;
+    UnicodeString id,s;
+    UBool startsInDST = tz.inDaylightTime(min, ec);
+    if (failure(ec, "TimeZone::inDaylightTime")) return;
+    if (tz.inDaylightTime(max, ec) == startsInDST) {
+        logln("Error: " + tz.getID(id) + ".inDaylightTime(" + dateToString(min) + ") = " + (startsInDST?"TRUE":"FALSE") +
+              ", inDaylightTime(" + dateToString(max) + ") = " + (startsInDST?"TRUE":"FALSE"));
+        return;
+    }
+    if (failure(ec, "TimeZone::inDaylightTime")) return;
+    while ((max - min) > INTERVAL) {
+        UDate mid = (min + max) / 2;
+        if (tz.inDaylightTime(mid, ec) == startsInDST) {
+            min = mid;
+        } else {
+            max = mid;
+        }
+        if (failure(ec, "TimeZone::inDaylightTime")) return;
+    }
+    min = 1000.0 * uprv_floor(min/1000.0);
+    max = 1000.0 * uprv_floor(max/1000.0);
+    logln(tz.getID(id) + " Before: " + min/1000 + " = " +
+          dateToString(min,s,tz));
+    logln(tz.getID(id) + " After:  " + max/1000 + " = " +
+          dateToString(max,s,tz));
+}
+
+void
+TimeZoneTest::testUsingBinarySearch(const TimeZone& tz,
+                                    UDate min, UDate max,
+                                    UDate expectedBoundary)
 {
     UErrorCode status = U_ZERO_ERROR;
-    UBool startsInDST = tz->inDaylightTime(min, status);
-    if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
-    if (tz->inDaylightTime(max, status) == startsInDST) {
+    UBool startsInDST = tz.inDaylightTime(min, status);
+    if (failure(status, "TimeZone::inDaylightTime")) return;
+    if (tz.inDaylightTime(max, status) == startsInDST) {
         logln("Error: inDaylightTime(" + dateToString(max) + ") != " + ((!startsInDST)?"TRUE":"FALSE"));
         return;
     }
-    if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
+    if (failure(status, "TimeZone::inDaylightTime")) return;
     while ((max - min) > INTERVAL) {
         UDate mid = (min + max) / 2;
-        if (tz->inDaylightTime(mid, status) == startsInDST) {
+        if (tz.inDaylightTime(mid, status) == startsInDST) {
             min = mid;
-        }
-        else {
+        } else {
             max = mid;
         }
-        if (failure(status, "SimpleTimeZone::inDaylightTime")) return;
+        if (failure(status, "TimeZone::inDaylightTime")) return;
     }
     logln(UnicodeString("Binary Search Before: ") + uprv_floor(0.5 + min) + " = " + dateToString(min));
     logln(UnicodeString("Binary Search After:  ") + uprv_floor(0.5 + max) + " = " + dateToString(max));
@@ -239,7 +280,15 @@ TimeZoneTest::TestPRTOffset()
         errln("FAIL: TimeZone(PRT) is null");
     }
     else {
-        if (tz->getRawOffset() != (- 4 * millisPerHour)) errln("FAIL: Offset for PRT should be -4");
+      int32_t expectedHour = -4;
+      double expectedOffset = (((double)expectedHour) * millisPerHour);
+      double foundOffset = tz->getRawOffset();
+      int32_t foundHour = (int32_t)foundOffset / millisPerHour;
+      if (expectedOffset != foundOffset) {
+        errln("FAIL: Offset for PRT should be %d, found %d", expectedHour, foundHour);
+      } else {
+        logln("PASS: Offset for PRT should be %d, found %d", expectedHour, foundHour);
+      }
     }
     delete tz;
 }
@@ -254,7 +303,7 @@ TimeZoneTest::TestVariousAPI518()
 {
     UErrorCode status = U_ZERO_ERROR;
     TimeZone* time_zone = TimeZone::createTimeZone("PST");
-    UDate d = date(97, Calendar::APRIL, 30);
+    UDate d = date(97, UCAL_APRIL, 30);
     UnicodeString str;
     logln("The timezone is " + time_zone->getID(str));
     if (!time_zone->inDaylightTime(d, status)) errln("FAIL: inDaylightTime returned FALSE");
@@ -265,8 +314,8 @@ TimeZoneTest::TestVariousAPI518()
     if (U_FAILURE(status)) { errln("FAIL: Couldn't create GregorianCalendar"); return; }
     gc->setTime(d, status);
     if (U_FAILURE(status)) { errln("FAIL: GregorianCalendar::setTime failed"); return; }
-    if (time_zone->getOffset(gc->AD, gc->get(gc->YEAR, status), gc->get(gc->MONTH, status),
-        gc->get(gc->DAY_OF_MONTH, status), (uint8_t)gc->get(gc->DAY_OF_WEEK, status), 0) != - 7 * millisPerHour)
+    if (time_zone->getOffset(gc->AD, gc->get(UCAL_YEAR, status), gc->get(UCAL_MONTH, status),
+        gc->get(UCAL_DATE, status), (uint8_t)gc->get(UCAL_DAY_OF_WEEK, status), 0, status) != - 7 * millisPerHour)
         errln("FAIL: getOffset returned wrong value");
     if (U_FAILURE(status)) { errln("FAIL: GregorianCalendar::set failed"); return; }
     delete gc;
@@ -281,14 +330,82 @@ TimeZoneTest::TestVariousAPI518()
 void
 TimeZoneTest::TestGetAvailableIDs913()
 {
-    UnicodeString str;
-    UnicodeString *buf = new UnicodeString("TimeZone.getAvailableIDs() = { ");
-    int32_t s_length;
-    const UnicodeString** s = TimeZone::createAvailableIDs(s_length);
+    UErrorCode ec = U_ZERO_ERROR;
     int32_t i;
+
+#ifdef U_USE_TIMEZONE_OBSOLETE_2_8
+    // Test legacy API -- remove these tests when the corresponding API goes away (duh)
+    int32_t numIDs = -1;
+    const UnicodeString** ids = TimeZone::createAvailableIDs(numIDs);
+    if (ids == 0 || numIDs < 1) {
+        errln("FAIL: createAvailableIDs()");
+    } else {
+        UnicodeString buf("TimeZone::createAvailableIDs() = { ");
+        for(i=0; i<numIDs; ++i) {
+            if (i) buf.append(", ");
+            buf.append(*ids[i]);
+        }
+        buf.append(" } ");
+        logln(buf + numIDs);
+        // we own the array; the caller owns the contained strings (yuck)
+        uprv_free(ids);
+    }
+
+    numIDs = -1;
+    ids = TimeZone::createAvailableIDs(-8*U_MILLIS_PER_HOUR, numIDs);
+    if (ids == 0 || numIDs < 1) {
+        errln("FAIL: createAvailableIDs(-8:00)");
+    } else {
+        UnicodeString buf("TimeZone::createAvailableIDs(-8:00) = { ");
+        for(i=0; i<numIDs; ++i) {
+            if (i) buf.append(", ");
+            buf.append(*ids[i]);
+        }
+        buf.append(" } ");
+        logln(buf + numIDs);
+        // we own the array; the caller owns the contained strings (yuck)
+        uprv_free(ids);
+    }
+    numIDs = -1;
+    ids = TimeZone::createAvailableIDs("US", numIDs);
+    if (ids == 0 || numIDs < 1) {
+      errln("FAIL: createAvailableIDs(US) ids=%d, numIDs=%d", ids, numIDs);
+    } else {
+        UnicodeString buf("TimeZone::createAvailableIDs(US) = { ");
+        for(i=0; i<numIDs; ++i) {
+            if (i) buf.append(", ");
+            buf.append(*ids[i]);
+        }
+        buf.append(" } ");
+        logln(buf + numIDs);
+        // we own the array; the caller owns the contained strings (yuck)
+        uprv_free(ids);
+    }
+#endif
+
+    UnicodeString str;
+    UnicodeString *buf = new UnicodeString("TimeZone::createEnumeration() = { ");
+    int32_t s_length;
+    StringEnumeration* s = TimeZone::createEnumeration();
+    s_length = s->count(ec);
     for (i = 0; i < s_length;++i) {
         if (i > 0) *buf += ", ";
-        *buf += *s[i];
+        if ((i & 1) == 0) {
+            *buf += *s->snext(ec);
+        } else {
+            *buf += UnicodeString(s->next(NULL, ec), "");
+        }
+
+        if((i % 5) == 4) {
+            // replace s with a clone of itself
+            StringEnumeration *s2 = s->clone();
+            if(s2 == NULL || s_length != s2->count(ec)) {
+                errln("TimezoneEnumeration.clone() failed");
+            } else {
+                delete s;
+                s = s2;
+            }
+        }
     }
     *buf += " };";
     logln(*buf);
@@ -297,33 +414,53 @@ TimeZoneTest::TestGetAvailableIDs913()
      * zone, the last zone, and one in-between.  This tests the binary
      * search through the system zone data.
      */
-    for (i=0; i<3; ++i) {
-        int32_t which = (i==0)?0:((i==1)?(s_length/2):(s_length-1));
-        TimeZone *z = TimeZone::createTimeZone(*s[which]);
+    s->reset(ec);
+    int32_t middle = s_length/2;
+    for (i=0; i<s_length; ++i) {
+        const UnicodeString* id = s->snext(ec);
+        if (i==0 || i==middle || i==(s_length-1)) {
+        TimeZone *z = TimeZone::createTimeZone(*id);
         if (z == 0) {
             errln(UnicodeString("FAIL: createTimeZone(") +
-                  *s[which] + ") -> 0");
-        } else if (z->getID(str) != *s[which]) {
+                  *id + ") -> 0");
+        } else if (z->getID(str) != *id) {
             errln(UnicodeString("FAIL: createTimeZone(") +
-                  *s[which] + ") -> zone " + str);
+                  *id + ") -> zone " + str);
         } else {
             logln(UnicodeString("OK: createTimeZone(") +
-                  *s[which] + ") succeeded");
+                  *id + ") succeeded");
         }
         delete z;
+        }
     }
-    delete [] s;
+    delete s;
 
     buf->truncate(0);
-    *buf += "TimeZone.getAvailableIDs(GMT+02:00) = { ";
+    *buf += "TimeZone::createEnumeration(GMT+01:00) = { ";
 
-    s = TimeZone::createAvailableIDs(+ 2 * 60 * 60 * 1000, s_length);
+    s = TimeZone::createEnumeration(1 * U_MILLIS_PER_HOUR);
+    s_length = s->count(ec);
     for (i = 0; i < s_length;++i) {
         if (i > 0) *buf += ", ";
-        *buf += *s[i];
+        *buf += *s->snext(ec);
+    }
+    delete s;
+    *buf += " };";
+    logln(*buf);
+
+
+    buf->truncate(0);
+    *buf += "TimeZone::createEnumeration(US) = { ";
+
+    s = TimeZone::createEnumeration("US");
+    s_length = s->count(ec);
+    for (i = 0; i < s_length;++i) {
+        if (i > 0) *buf += ", ";
+        *buf += *s->snext(ec);
     }
     *buf += " };";
     logln(*buf);
+
     TimeZone *tz = TimeZone::createTimeZone("PST");
     if (tz != 0) logln("getTimeZone(PST) = " + tz->getID(str));
     else errln("FAIL: getTimeZone(PST) = null");
@@ -343,12 +480,15 @@ TimeZoneTest::TestGetAvailableIDs913()
     delete tz;
 
     delete buf;
-    delete [] s;
-
+    delete s;
 }
 
 
 /**
+ * NOTE: As of ICU 2.8, this test confirms that the "tz.alias"
+ * file, used to build ICU alias zones, is working.  It also
+ * looks at some genuine Olson compatibility IDs. [aliu]
+ *
  * This test is problematic. It should really just confirm that
  * the list of compatibility zone IDs exist and are somewhat
  * meaningful (that is, they aren't all aliases of GMT). It goes a
@@ -437,10 +577,10 @@ void TimeZoneTest::TestShortZoneIDs()
         {"AST", -540, TRUE},
         {"PST", -480, TRUE},
         {"PNT", -420, FALSE},
-        {"MST", -420, TRUE},
+        {"MST", -420, FALSE}, // updated Aug 2003 aliu
         {"CST", -360, TRUE},
-        {"IET", -300, FALSE},
-        {"EST", -300, TRUE},
+        {"IET", -300, TRUE},  // updated Jan 2006 srl
+        {"EST", -300, FALSE}, // updated Aug 2003 aliu
         {"PRT", -240, FALSE},
         {"CNT", -210, TRUE},
         {"AGT", -180, FALSE}, // updated 26 Sep 2000 aliu
@@ -456,13 +596,13 @@ void TimeZoneTest::TestShortZoneIDs()
         {"EAT", 180, FALSE},
         {"MET", 60, TRUE}, // updated 12/3/99 aliu
         {"NET", 240, TRUE}, // updated 12/3/99 aliu
-        {"PLT", 300, FALSE},
+        {"PLT", 300, FALSE}, // updated Aug 2003 aliu
         {"IST", 330, FALSE},
         {"BST", 360, FALSE},
         {"VST", 420, FALSE},
-        {"CTT", 480, TRUE}, // updated 12/3/99 aliu
+        {"CTT", 480, FALSE}, // updated Aug 2003 aliu
         {"JST", 540, FALSE},
-        {"ACT", 570, TRUE}, // updated 12/3/99 aliu
+        {"ACT", 570, FALSE}, // updated Aug 2003 aliu
         {"AET", 600, TRUE},
         {"SST", 660, FALSE},
         // "NST", 720, FALSE,
@@ -514,7 +654,7 @@ void TimeZoneTest::TestShortZoneIDs()
 
     const char* compatibilityMap[] = {
         // This list is copied from tz.alias.  If tz.alias
-        // changes, this list must be updated.  Current as of 1/31/01
+        // changes, this list must be updated.  Current as of Aug 2003
         "ACT", "Australia/Darwin",
         "AET", "Australia/Sydney",
         "AGT", "America/Buenos_Aires",
@@ -529,14 +669,14 @@ void TimeZoneTest::TestShortZoneIDs()
         "EAT", "Africa/Addis_Ababa",
         "ECT", "Europe/Paris",
         // EET Europe/Istanbul # EET is a standard UNIX zone
-        "EST", "America/New_York",
+        // "EST", "America/New_York", # EST is an Olson alias now (2003)
         "HST", "Pacific/Honolulu",
         "IET", "America/Indianapolis",
         "IST", "Asia/Calcutta",
         "JST", "Asia/Tokyo",
         // MET Asia/Tehran # MET is a standard UNIX zone
         "MIT", "Pacific/Apia",
-        "MST", "America/Denver",
+        // "MST", "America/Denver", # MST is an Olson alias now (2003)
         "NET", "Asia/Yerevan",
         "NST", "Pacific/Auckland",
         "PLT", "Asia/Karachi",
@@ -637,8 +777,8 @@ void TimeZoneTest::TestCustomParse()
         // ID        Expected offset in minutes
         //"GMT",       kUnparseable,   Isn't custom. Can't test it here. [returns normal GMT]
         {"GMT-YOUR.AD.HERE", kUnparseable},
-        {"GMT0",      kUnparseable},
-        {"GMT+0",     (0)},
+        // {"GMT0",      kUnparseable}, // ICU 2.8: An Olson zone ID
+        // {"GMT+0",     (0)}, // ICU 2.8: An Olson zone ID
         {"GMT+1",     (60)},
         {"GMT-0030",  (-30)},
         {"GMT+15:99", (15*60+99)},
@@ -659,6 +799,17 @@ void TimeZoneTest::TestCustomParse()
     {
         UnicodeString id(kData[i].customId);
         int32_t exp = kData[i].expectedOffset;
+/*
+        { // for no data test Jitterbug 4354
+            UErrorCode success = U_ZERO_ERROR;
+            NumberFormat* numberFormat = NumberFormat::createInstance(success);
+            if (U_FAILURE(success)) {
+                dataerrln(" NumberFormat::createInstance() error");
+                return;
+            }
+            delete numberFormat;
+        }
+        */
 
         TimeZone *zone = TimeZone::createTimeZone(id);
         UnicodeString   itsID, temp;
@@ -731,7 +882,7 @@ TimeZoneTest::TestDisplayName()
     int32_t i;
     TimeZone *zone = TimeZone::createTimeZone("PST");
     UnicodeString name;
-    zone->getDisplayName(Locale::ENGLISH, name);
+    zone->getDisplayName(Locale::getEnglish(), name);
     logln("PST->" + name);
     if (name.compare("Pacific Standard Time") != 0)
         errln("Fail: Expected \"Pacific Standard Time\" but got " + name);
@@ -760,60 +911,75 @@ TimeZoneTest::TestDisplayName()
         name.remove();
         name = zone->getDisplayName(kData[i].useDst,
                                    kData[i].style,
-                                   Locale::ENGLISH, name);
+                                   Locale::getEnglish(), name);
+        if (name.compare(kData[i].expect) != 0)
+            errln("Fail: Expected " + UnicodeString(kData[i].expect) + "; got " + name);
+        logln("PST [with options]->" + name);
+    }
+    for (i=0; kData[i].expect[0] != '\0'; i++)
+    {
+        name.remove();
+        name = zone->getDisplayName(kData[i].useDst,
+                                   kData[i].style, name);
         if (name.compare(kData[i].expect) != 0)
             errln("Fail: Expected " + UnicodeString(kData[i].expect) + "; got " + name);
         logln("PST [with options]->" + name);
     }
 
+
     // Make sure that we don't display the DST name by constructing a fake
     // PST zone that has DST all year long.
     SimpleTimeZone *zone2 = new SimpleTimeZone(0, "PST");
 
-    zone2->setStartRule(Calendar::JANUARY, 1, 0, 0, status);
-    zone2->setEndRule(Calendar::DECEMBER, 31, 0, 0, status);
+    zone2->setStartRule(UCAL_JANUARY, 1, 0, 0, status);
+    zone2->setEndRule(UCAL_DECEMBER, 31, 0, 0, status);
 
-    UnicodeString inDaylight = (zone2->inDaylightTime(UDate(0), status)? UnicodeString("TRUE"):UnicodeString("FALSE"));
+    UnicodeString inDaylight;
+    if (zone2->inDaylightTime(UDate(0), status)) {
+        inDaylight = UnicodeString("TRUE");
+    } else {
+        inDaylight = UnicodeString("FALSE");
+    }
     logln(UnicodeString("Modified PST inDaylightTime->") + inDaylight );
     if(U_FAILURE(status))
     {
         errln("Some sort of error..." + UnicodeString(u_errorName(status))); // REVISIT
     }
     name.remove();
-    name = zone2->getDisplayName(Locale::ENGLISH,name);
+    name = zone2->getDisplayName(Locale::getEnglish(),name);
     logln("Modified PST->" + name);
     if (name.compare("Pacific Standard Time") != 0)
         errln("Fail: Expected \"Pacific Standard Time\"");
 
     // Make sure we get the default display format for Locales
     // with no display name data.
-    Locale zh_CN = Locale::SIMPLIFIED_CHINESE;
+    Locale mt_MT("mt_MT");
     name.remove();
-    name = zone->getDisplayName(zh_CN,name);
+    name = zone->getDisplayName(mt_MT,name);
     //*****************************************************************
     // THE FOLLOWING LINE MUST BE UPDATED IF THE LOCALE DATA CHANGES
     // THE FOLLOWING LINE MUST BE UPDATED IF THE LOCALE DATA CHANGES
     // THE FOLLOWING LINE MUST BE UPDATED IF THE LOCALE DATA CHANGES
     //*****************************************************************
-    logln("PST(zh_CN)->" + name);
+    logln("PST(mt_MT)->" + name);
 
     // *** REVISIT SRL how in the world do I check this? looks java specific.
     // Now be smart -- check to see if zh resource is even present.
     // If not, we expect the en fallback behavior.
-    ResourceBundle enRB(u_getDataDirectory(),
-                            Locale::ENGLISH, status);
+    ResourceBundle enRB(NULL,
+                            Locale::getEnglish(), status);
     if(U_FAILURE(status))
         errln("Couldn't get ResourceBundle for en");
 
-    ResourceBundle zhRB(u_getDataDirectory(),
-                         zh_CN, status);
+    ResourceBundle mtRB(NULL,
+                         mt_MT, status);
     //if(U_FAILURE(status))
-    //    errln("Couldn't get ResourceBundle for zh_CN");
+    //    errln("Couldn't get ResourceBundle for mt_MT");
 
     UBool noZH = U_FAILURE(status);
 
     if (noZH) {
-        logln("Warning: Not testing the zh_CN behavior because resource is absent");
+        logln("Warning: Not testing the mt_MT behavior because resource is absent");
         if (name != "Pacific Standard Time")
             errln("Fail: Expected Pacific Standard Time");
     }
@@ -823,7 +989,7 @@ TimeZoneTest::TestDisplayName()
              name.compare("GMT-8:00") &&
              name.compare("GMT-0800") &&
              name.compare("GMT-800")) {
-        errln("Fail: Expected GMT-08:00 or something similar");
+      errln(UnicodeString("Fail: Expected GMT-08:00 or something similar for PST in mt_MT but got ") + name );
         errln("************************************************************");
         errln("THE ABOVE FAILURE MAY JUST MEAN THE LOCALE DATA HAS CHANGED");
         errln("************************************************************");
@@ -833,14 +999,21 @@ TimeZoneTest::TestDisplayName()
     delete zone2;
     zone2 = new SimpleTimeZone(90*60*1000, "xyzzy");
     name.remove();
-    name = zone2->getDisplayName(Locale::ENGLISH,name);
+    name = zone2->getDisplayName(Locale::getEnglish(),name);
+    logln("GMT+90min->" + name);
+    if (name.compare("GMT+01:30") &&
+        name.compare("GMT+1:30") &&
+        name.compare("GMT+0130") &&
+        name.compare("GMT+130"))
+        errln("Fail: Expected GMT+01:30 or something similar");
+    name.truncate(0);
+    zone2->getDisplayName(name);
     logln("GMT+90min->" + name);
     if (name.compare("GMT+01:30") &&
         name.compare("GMT+1:30") &&
         name.compare("GMT+0130") &&
         name.compare("GMT+130"))
         errln("Fail: Expected GMT+01:30 or something similar");
-
     // clean up
     delete zone;
     delete zone2;
@@ -857,7 +1030,7 @@ TimeZoneTest::TestDSTSavings()
     // tests above, but I don't have time to figure out how to do this (or if it's
     // even really a good idea).  Let's consider that a future.  --rtg 1/27/98
     SimpleTimeZone *tz = new SimpleTimeZone(-5 * U_MILLIS_PER_HOUR, "dstSavingsTest",
-                                           Calendar::MARCH, 1, 0, 0, Calendar::SEPTEMBER, 1, 0, 0,
+                                           UCAL_MARCH, 1, 0, 0, UCAL_SEPTEMBER, 1, 0, 0,
                                            (int32_t)(0.5 * U_MILLIS_PER_HOUR), status);
     if(U_FAILURE(status))
         errln("couldn't create TimeZone");
@@ -871,27 +1044,27 @@ TimeZoneTest::TestDSTSavings()
         errln(UnicodeString("Set DST offset to 0.5 hour, but got back ") + (tz->getDSTSavings() /
                                                              U_MILLIS_PER_HOUR) + " hours instead.");
 
-    int32_t offset = tz->getOffset(GregorianCalendar::AD, 1998, Calendar::JANUARY, 1,
-                              Calendar::THURSDAY, 10 * U_MILLIS_PER_HOUR);
+    int32_t offset = tz->getOffset(GregorianCalendar::AD, 1998, UCAL_JANUARY, 1,
+                              UCAL_THURSDAY, 10 * U_MILLIS_PER_HOUR,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz->getOffset(GregorianCalendar::AD, 1998, Calendar::JUNE, 1, Calendar::MONDAY,
-                          10 * U_MILLIS_PER_HOUR);
+    offset = tz->getOffset(GregorianCalendar::AD, 1998, UCAL_JUNE, 1, UCAL_MONDAY,
+                          10 * U_MILLIS_PER_HOUR,status);
     if (offset != -4.5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10 AM, 6/1/98 should have been -4.5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
     tz->setDSTSavings(U_MILLIS_PER_HOUR, status);
-    offset = tz->getOffset(GregorianCalendar::AD, 1998, Calendar::JANUARY, 1,
-                          Calendar::THURSDAY, 10 * U_MILLIS_PER_HOUR);
+    offset = tz->getOffset(GregorianCalendar::AD, 1998, UCAL_JANUARY, 1,
+                          UCAL_THURSDAY, 10 * U_MILLIS_PER_HOUR,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10 AM, 1/1/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz->getOffset(GregorianCalendar::AD, 1998, Calendar::JUNE, 1, Calendar::MONDAY,
-                          10 * U_MILLIS_PER_HOUR);
+    offset = tz->getOffset(GregorianCalendar::AD, 1998, UCAL_JUNE, 1, UCAL_MONDAY,
+                          10 * U_MILLIS_PER_HOUR,status);
     if (offset != -4 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10 AM, 6/1/98 (with a 1-hour DST offset) should have been -4 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
@@ -913,64 +1086,64 @@ TimeZoneTest::TestAlternateRules()
 
     // test the day-of-month API
     UErrorCode status = U_ZERO_ERROR;
-    tz.setStartRule(Calendar::MARCH, 10, 12 * U_MILLIS_PER_HOUR, status);
+    tz.setStartRule(UCAL_MARCH, 10, 12 * U_MILLIS_PER_HOUR, status);
     if(U_FAILURE(status))
         errln("tz.setStartRule failed");
-    tz.setEndRule(Calendar::OCTOBER, 20, 12 * U_MILLIS_PER_HOUR, status);
+    tz.setEndRule(UCAL_OCTOBER, 20, 12 * U_MILLIS_PER_HOUR, status);
     if(U_FAILURE(status))
         errln("tz.setStartRule failed");
 
-    int32_t offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::MARCH, 5,
-                              Calendar::THURSDAY, 10 * U_MILLIS_PER_HOUR);
+    int32_t offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_MARCH, 5,
+                              UCAL_THURSDAY, 10 * U_MILLIS_PER_HOUR,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 3/5/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::MARCH, 15,
-                          Calendar::SUNDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_MARCH, 15,
+                          UCAL_SUNDAY, 10 * millisPerHour,status);
     if (offset != -4 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 3/15/98 should have been -4 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::OCTOBER, 15,
-                          Calendar::THURSDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_OCTOBER, 15,
+                          UCAL_THURSDAY, 10 * millisPerHour,status);
     if (offset != -4 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 10/15/98 should have been -4 hours, but we got ")              + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::OCTOBER, 25,
-                          Calendar::SUNDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_OCTOBER, 25,
+                          UCAL_SUNDAY, 10 * millisPerHour,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 10/25/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
     // test the day-of-week-after-day-in-month API
-    tz.setStartRule(Calendar::MARCH, 10, Calendar::FRIDAY, 12 * millisPerHour, TRUE, status);
+    tz.setStartRule(UCAL_MARCH, 10, UCAL_FRIDAY, 12 * millisPerHour, TRUE, status);
     if(U_FAILURE(status))
         errln("tz.setStartRule failed");
-    tz.setEndRule(Calendar::OCTOBER, 20, Calendar::FRIDAY, 12 * millisPerHour, FALSE, status);
+    tz.setEndRule(UCAL_OCTOBER, 20, UCAL_FRIDAY, 12 * millisPerHour, FALSE, status);
     if(U_FAILURE(status))
         errln("tz.setStartRule failed");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::MARCH, 11,
-                          Calendar::WEDNESDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_MARCH, 11,
+                          UCAL_WEDNESDAY, 10 * millisPerHour,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 3/11/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::MARCH, 14,
-                          Calendar::SATURDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_MARCH, 14,
+                          UCAL_SATURDAY, 10 * millisPerHour,status);
     if (offset != -4 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 3/14/98 should have been -4 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::OCTOBER, 15,
-                          Calendar::THURSDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_OCTOBER, 15,
+                          UCAL_THURSDAY, 10 * millisPerHour,status);
     if (offset != -4 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 10/15/98 should have been -4 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
 
-    offset = tz.getOffset(GregorianCalendar::AD, 1998, Calendar::OCTOBER, 17,
-                          Calendar::SATURDAY, 10 * millisPerHour);
+    offset = tz.getOffset(GregorianCalendar::AD, 1998, UCAL_OCTOBER, 17,
+                          UCAL_SATURDAY, 10 * millisPerHour,status);
     if (offset != -5 * U_MILLIS_PER_HOUR)
         errln(UnicodeString("The offset for 10AM, 10/17/98 should have been -5 hours, but we got ")
               + (offset / U_MILLIS_PER_HOUR) + " hours.");
@@ -982,22 +1155,25 @@ TimeZoneTest::TestAlternateRules()
 void TimeZoneTest::TestCountries() {
     // Make sure America/Los_Angeles is in the "US" group, and
     // Asia/Tokyo isn't.  Vice versa for the "JP" group.
+    UErrorCode ec = U_ZERO_ERROR;
     int32_t n;
-    const UnicodeString** s = TimeZone::createAvailableIDs("US", n);
+    StringEnumeration* s = TimeZone::createEnumeration("US");
+    n = s->count(ec);
     UBool la = FALSE, tokyo = FALSE;
     UnicodeString laZone("America/Los_Angeles", "");
     UnicodeString tokyoZone("Asia/Tokyo", "");
     int32_t i;
 
     if (s == NULL || n <= 0) {
-        errln("FAIL: TimeZone::createAvailableIDs() returned nothing");
+        errln("FAIL: TimeZone::createEnumeration() returned nothing");
         return;
     }
     for (i=0; i<n; ++i) {
-        if (*s[i] == (laZone)) {
+        const UnicodeString* id = s->snext(ec);
+        if (*id == (laZone)) {
             la = TRUE;
         }
-        if (*s[i] == (tokyoZone)) {
+        if (*id == (tokyoZone)) {
             tokyo = TRUE;
         }
     }
@@ -1005,16 +1181,18 @@ void TimeZoneTest::TestCountries() {
         errln("FAIL: " + laZone + " in US = " + la);
         errln("FAIL: " + tokyoZone + " in US = " + tokyo);
     }
-    delete[] s;
+    delete s;
     
-    s = TimeZone::createAvailableIDs("JP", n);
+    s = TimeZone::createEnumeration("JP");
+    n = s->count(ec);
     la = FALSE; tokyo = FALSE;
     
     for (i=0; i<n; ++i) {
-        if (*s[i] == (laZone)) {
+        const UnicodeString* id = s->snext(ec);
+        if (*id == (laZone)) {
             la = TRUE;
         }
-        if (*s[i] == (tokyoZone)) {
+        if (*id == (tokyoZone)) {
             tokyo = TRUE;
         }
     }
@@ -1022,5 +1200,104 @@ void TimeZoneTest::TestCountries() {
         errln("FAIL: " + laZone + " in JP = " + la);
         errln("FAIL: " + tokyoZone + " in JP = " + tokyo);
     }
-    delete[] s;
+    StringEnumeration* s1 = TimeZone::createEnumeration("US");
+    StringEnumeration* s2 = TimeZone::createEnumeration("US");
+    for(i=0;i<n;++i){
+        const UnicodeString* id1 = s1->snext(ec);
+        if(id1==NULL || U_FAILURE(ec)){
+            errln("Failed to fetch next from TimeZone enumeration. Length returned : %i Current Index: %i", n,i);
+        }
+        TimeZone* tz1 = TimeZone::createTimeZone(*id1);
+        for(int j=0; j<n;++j){
+            const UnicodeString* id2 = s2->snext(ec);
+            if(id2==NULL || U_FAILURE(ec)){
+                errln("Failed to fetch next from TimeZone enumeration. Length returned : %i Current Index: %i", n,i);
+            }
+            TimeZone* tz2 = TimeZone::createTimeZone(*id2);
+            if(tz1->hasSameRules(*tz2)){
+                logln("ID1 : " + *id1+" == ID2 : " +*id2);
+            }
+            delete tz2;
+        }
+        delete tz1;
+    }
+    delete s1;
+    delete s2;
+    delete s;
 }
+
+void TimeZoneTest::TestHistorical() {
+    const int32_t H = U_MILLIS_PER_HOUR;
+    struct {
+        const char* id;
+        int32_t time; // epoch seconds
+        int32_t offset; // total offset (millis)
+    } DATA[] = {
+        // Add transition points (before/after) as desired to test historical
+        // behavior.
+        {"America/Los_Angeles", 638963999, -8*H}, // Sun Apr 01 01:59:59 GMT-08:00 1990
+        {"America/Los_Angeles", 638964000, -7*H}, // Sun Apr 01 03:00:00 GMT-07:00 1990
+        {"America/Los_Angeles", 657104399, -7*H}, // Sun Oct 28 01:59:59 GMT-07:00 1990
+        {"America/Los_Angeles", 657104400, -8*H}, // Sun Oct 28 01:00:00 GMT-08:00 1990
+        {"America/Goose_Bay", -116445601, -4*H}, // Sun Apr 24 01:59:59 GMT-04:00 1966
+        {"America/Goose_Bay", -116445600, -3*H}, // Sun Apr 24 03:00:00 GMT-03:00 1966
+        {"America/Goose_Bay", -100119601, -3*H}, // Sun Oct 30 01:59:59 GMT-03:00 1966
+        {"America/Goose_Bay", -100119600, -4*H}, // Sun Oct 30 01:00:00 GMT-04:00 1966
+        {"America/Goose_Bay", -84391201, -4*H}, // Sun Apr 30 01:59:59 GMT-04:00 1967
+        {"America/Goose_Bay", -84391200, -3*H}, // Sun Apr 30 03:00:00 GMT-03:00 1967
+        {"America/Goose_Bay", -68670001, -3*H}, // Sun Oct 29 01:59:59 GMT-03:00 1967
+        {"America/Goose_Bay", -68670000, -4*H}, // Sun Oct 29 01:00:00 GMT-04:00 1967
+        {0, 0, 0}
+    };
+    
+    for (int32_t i=0; DATA[i].id!=0; ++i) {
+        const char* id = DATA[i].id;
+        TimeZone *tz = TimeZone::createTimeZone(id);
+        UnicodeString s;
+        if (tz == 0) {
+            errln("FAIL: Cannot create %s", id);
+        } else if (tz->getID(s) != UnicodeString(id)) {
+            errln((UnicodeString)"FAIL: createTimeZone(" + id + ") => " + s);
+        } else {
+            UErrorCode ec = U_ZERO_ERROR;
+            int32_t raw, dst;
+            UDate when = (double) DATA[i].time * U_MILLIS_PER_SECOND;
+            tz->getOffset(when, FALSE, raw, dst, ec);
+            if (U_FAILURE(ec)) {
+                errln("FAIL: getOffset");
+            } else if ((raw+dst) != DATA[i].offset) {
+                errln((UnicodeString)"FAIL: " + DATA[i].id + ".getOffset(" +
+                      //when + " = " +
+                      dateToString(when) + ") => " +
+                      raw + ", " + dst);
+            } else {
+                logln((UnicodeString)"Ok: " + DATA[i].id + ".getOffset(" +
+                      //when + " = " +
+                      dateToString(when) + ") => " +
+                      raw + ", " + dst);
+            }
+        }
+        delete tz;
+    }
+}
+
+void TimeZoneTest::TestEquivalentIDs() {
+    int32_t n = TimeZone::countEquivalentIDs("PST");
+    if (n < 2) {
+        errln((UnicodeString)"FAIL: countEquivalentIDs(PST) = " + n);
+    } else {
+        UBool sawLA = FALSE;
+        for (int32_t i=0; i<n; ++i) {
+            UnicodeString id = TimeZone::getEquivalentID("PST", i);
+            logln((UnicodeString)"" + i + " : " + id);
+            if (id == UnicodeString("America/Los_Angeles")) {
+                sawLA = TRUE;
+            }
+        }
+        if (!sawLA) {
+            errln("FAIL: America/Los_Angeles should be in the list");
+        }
+    }
+}
+
+#endif /* #if !UCONFIG_NO_FORMATTING */
